//STRIPPEDSOURCE//

package com.ms.wfc.core;

import com.ms.wfc.util.*;
import java.lang.reflect.*;
import com.ms.lang.Delegate;

/**
 * Defines an event of a component.  Events are the main way that a user can get 
 * run-time notifications from a component. 
 * <p>
 * The EventInfo class takes a component class that the event lives on,
 * the event name, the type of the event handling delegate, and various 
 * attributes for the event.
 * <p>
 * Every event has a structure through which it passes it's information.  The base
 * structure, Event, is empty, and there is a default instance, Event.EMPTY, which
 * is usually passed.  When addOnXXX is invoked, it needs a pointer to a method
 * that takes a source object (the object that fired the event) and a structure
 * particular to that event.  It also needs a pointer to the instance of the 
 * object the method should be invoked on. These two things are what composes a 
 * delegate.  An event handler is
 * a delegate, and the compiler recognizes a special delegate syntax that makes
 * using delegates easy.
 * <p>
 * For example, to listen to the click event on a button in class Foo, the 
 * following code will  suffice:
 * <pre>
 *   class Foo {    
 *     Button button1 = new Button();
 * 
 *     void button1_click(Object sender, Event e) {
 *        // do something on button1 click.
 *     }
 *  
 *     public Foo() {
 *       button1.addOnClick(this.button1_click);
 *     }    
 *   }
 * </pre>
 * For an event named XXX, a YYYEvent structure, and a YYYEventHandler delegate,
 * a component writer is required to implement two methods of the following 
 * form:
 * <pre>
 *   public void addOnXXX(YYYEventHandler handler);
 *   public void removeOnXXX(YYYEventHandler handler);
 * </pre>
 * YYYEventHandler should be an event handler declared as
 * <pre>
 * public multicast delegate void YYYEventHandler(Object sender, YYYEvent e);
 * </pre>
 * Note that this event was declared as a multicast delegate.  This allows multiple
 * listeners on an event.  This is not a requirement.
 * <p>
 * Various attributes can be passed to the EventInfo, as are described in
 * MemberAttribute.
 * <p>
 * EventInfos can be obtained by a user programmatically through the
 * ComponentManager.
 * 
 * @see IClassInfo
 * @see MemberInfo
 * @see MemberAttribute
 * @see ComponentManager
 *
 * @security(checkClassLinking=on)
 */
public final class EventInfo extends MemberInfo { 
    static final Class[] argsNone = new Class[0];
    static final Object noDefault = new Object();

    private final Class type;
    private final Method addMethod;
    private final Method removeMethod;

    /**
     * This is the main constructor for an EventInfo.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     * @param attributes attributes for this event.
     */
    public EventInfo(Class componentClass, String name, Class type,
        MemberAttribute[] attributes) {}

    /**
     * This is a shortcut main constructor for an EventInfo with one attribute.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     */
    public EventInfo(Class componentClass, String name, Class type) {}

    /**
     * This is a shortcut main constructor for an EventInfo with two attributes.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     * @param a1 An attribute for this event.
     */
    public EventInfo(Class componentClass, String name, Class type,
        MemberAttribute a1) {}

    /**
     * This is a shortcut main constructor for an EventInfo with two attributes.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     * @param a1 An attribute for this event.
     * @param a2 An attribute for this event.
     */
    public EventInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2) {}

    /**
     * This is a shortcut main constructor for an EventInfo with three attributes.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     * @param a1 An attribute for this event.
     * @param a2 An attribute for this event.
     * @param a3 An attribute for this event.
     */
    public EventInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2, MemberAttribute a3) {}

    /**
     * This is a shortcut main constructor for an EventInfo with four attributes.
     * 
     * @param componentClass The class of the component the event lives on.
     * @param name The name of the event.
     * @param type The type of the event handling delegate.
     * @param a1 An attribute for this event.
     * @param a2 An attribute for this event.
     * @param a3 An attribute for this event.
     * @param a4 An attribute for this event.
     */
    public EventInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2,
        MemberAttribute a3, MemberAttribute a4) {}

    /**
     * This constructor takes an existing EventInfo and modifies it by merging in the
     * passed-in attributes.
     * 
     * @param oldEventInfo The existing event info.
     * @param attributes The new attributes.
     */
    public EventInfo(EventInfo oldEventInfo, MemberAttribute[] attributes) {}

    /**
     * This is a shortcut constructor that takes an existing EventInfo and one attribute to
     * merge in.
     * 
     * @param oldEventInfo The existing event info.
     * @param a1 A new attribute.
     */
    public EventInfo(EventInfo oldEventInfo, MemberAttribute a1) {}

    /**
     * This is a shortcut constructor that takes an existing EventInfo and two attributes to
     * merge in.
     * 
     * @param oldEventInfo The existing event info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     */
    public EventInfo(EventInfo oldEventInfo, MemberAttribute a1,
        MemberAttribute a2) {}

    /**
     * This is a shortcut constructor that takes an existing EventInfo and three attributes to
     * merge in.
     * 
     * @param oldEventInfo The existing event info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     */
    public EventInfo(EventInfo oldEventInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3) {}

    /**
     * This is a shortcut constructor that takes an existing EventInfo and four attributes to
     * merge in.
     * 
     * @param oldEventInfo The existing event info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     * @param a4 A new attribute.
     */
    public EventInfo(EventInfo oldEventInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3, MemberAttribute a4) {}

    /**
     * Adds a delegate to a component. 
     * 
     * @param component The component on which the delegate wants to hear
     * events from.
     * @param value The delegate which encapsulates the listening object 
     * and method.
     */
    public void addEventHandler(Object component, Delegate value) {}

    /**
     * Retrieves the addOn<EventName> method of the component class. 
     * 
     * @return the addOnXXX method pointer.
     */
    public Method getAddMethod() {}

    /**
     * Defines an event of a component. 
     * 
     * @return the type of the delegate for this event.
     */
    public Class getType() {}

    /**
     * Retrieves the removeOn<EventName> method of the component class. 
     * 
     * @return the removeOnXXX method pointer.
     */
    public Method getRemoveMethod() {}

    /**
     * Determines whether the delegate type for the event is declared as multicast.
     * 
     * 
     * @return whether the delegate is multicast.
     */
    public boolean getMulticast() {}

    /**
     * Removes a delegate from a component.
     * 
     * @param component from which the delegate will be removed.
     * @param delegate to remove.
     */
    public void removeEventHandler(Object component, Delegate value) {}
}