//STRIPPEDSOURCE//

package com.ms.wfc.core;

import com.ms.wfc.util.*;
import java.lang.reflect.*;

/**
 * Defines an extender property of a component.  A component that implements 
 * IExtenderProvider can proffer extender properties to other components.
 * The goal of extender properties is to make it appear that components have
 * new and extra functionality by adding extenders to the same container as
 * these components. An example is the Tooltip component, which can offer 
 * tooltips for all other controls on a form. A tool may make it appear that
 * the controls all have an extra property "tooltip" when this component is
 * present.
 * <p>
 * The ExtenderInfo class takes a component class that the extender is offered 
 * from, the extender name, the type of the extender, and various attributes 
 * for the extender.
 * <p>
 * For an extender named XXX of type YYY, the associated component class is 
 * required to implement two methods of the following 
 * form:
 * <pre>
 *   public YYY getXXX(Object extendee);
 *   public void setXXX(Object extendee, YYY value);
 * </pre>
 * Here, the argument extendee is an object that the component proffers the
 * extender property for.  It can look in a tool as if the extendee object
 * has an additional property XXX.
 * <p>
 * For example, given a Form with button1 and button2 on it, tooltips can
 * be provided for button1 and button2 by tooltip1 as follows:
 * <pre>
 *   Button button1 = new Button();
 *   Button button2 = new Button();
 *   Tooltip tooltip1 = new Tooltip();
 *   
 *   tooltip1.setTooltip(button1,"Button1");
 *   tooltip1.setTooltip(button2,"Button2");
 * </pre>
 * An extender object must implement IExtenderProvider to proffer extender 
 * properties. The only function canExtend(Object extendee) on IExtenderProvider which
 * allows extenders to decide on an instance basis what components to proffer
 * properties to.
 * <p>
 * The component class can optionally implement two additional methods of 
 * the following form:
 * <pre>
 *   public boolean shouldPersistXXX(Object extendee);
 *   public void resetXXX(Object extendee);
 * </pre>
 * These methods deal with a property's default value. The shouldPersistXXX
 * method returns true if the current value of the XXX extender for the extendee
 * is different than it's default value, so that it should be persisted out.  The 
 * resetXXX method resets the XXX extender for the extendee to its default 
 * value. If the ExtenderInfo includes the default value of the property 
 * (using the DefaultValueAttribute), the shouldPersistXXX and resetXXX 
 * methods are ignored. 
 * <p>
 * If the ExtenderInfo includes a reference to a value editor
 * (the ValueEditorAttribute), then that value editor will be used to 
 * edit the property. Otherwise, a system-provided value editor will be used.
 * <p>
 * Various attributes can be passed to the ExtenderInfo, as are described in
 * MemberAttribute.
 * 
 * @see IClassInfo
 * @see MemberInfo
 * @see MemberAttribute
 * @see DefaultValueAttribute
 * @see ValueEditorAttribute
 *
 * @security(checkClassLinking=on)
 */
public class ExtenderInfo extends MemberInfo { 
    static final Object noDefault = new Object();
    
    private Class extendeeType;
    private Class propertyType;
    private Object defaultValue;
    private Method getMethod;
    private Method setMethod;
    private Method shouldPersistMethod;
    private Method resetMethod;
    
    /**
     * The main constructor for ExtenderInfo. 
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param propertyName The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     * @param attributes The attributes of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeClass, Class propertyType,
        MemberAttribute[] attributes) {}
    
    /**
     * The shortcut constructor for ExtenderInfos with no attributes.
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param name The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeType, Class propertyType) {}
    
    /**
     * The shorcut constructor for ExtenderInfos with one attribute.
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param name The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     * @param a1 An attribute of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeType, Class propertyType,
        MemberAttribute a1) {}
    
    /**
     * The shorcut constructor for ExtenderInfos with two attributes.
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param name The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeType, Class propertyType,
        MemberAttribute a1, MemberAttribute a2) {}
    
    /**
     * The shorcut constructor for ExtenderInfos with three attributes.
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param name The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeType, Class propertyType,
        MemberAttribute a1, MemberAttribute a2, MemberAttribute a3) {}
    
    /**
     * The shorcut constructor for ExtenderInfos with four attributes.
     * 
     * @param extenderClass The type of the extender. (should implement IExtenderProvider).
     * @param name The name of the property.
     * @param extendeeClass The type of the extendee (The receiver of the extender property).
     * @param propertyType The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     * @param a4 An attribute of the property.
     */
    public ExtenderInfo(Class extenderClass, String propertyName, Class extendeeType, Class propertyType,
        MemberAttribute a1, MemberAttribute a2,
        MemberAttribute a3, MemberAttribute a4) {}
    
    /**
     * This constructor takes an existing ExtenderInfo and modifies it by merging in the
     * passed-in attributes.
     * 
     * @param oldExtenderInfo The existing extender info.
     * @param attributes The new attributes.
     */
    public ExtenderInfo(ExtenderInfo oldExtenderInfo, MemberAttribute[] attributes) {}
    
    /**
     * This is a shortcut constructor that takes an existing ExtenderInfo and one attribute to
     *  merge in.
     * 
     * @param oldExtenderInfo The existing extender info.
     * @param a1 A new attribute.
     */
    public ExtenderInfo(ExtenderInfo oldExtenderInfo, MemberAttribute a1) {}
    
    /**
     * This is a shortcut constructor that takes an existing ExtenderInfo and two attributes to
     *  merge in.
     * 
     * @param oldExtenderInfo The existing extender info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     */    
    public ExtenderInfo(ExtenderInfo oldExtenderInfo, MemberAttribute a1,
        MemberAttribute a2) {}
    
    /**
     * This is a shortcut constructor that takes an existing ExtenderInfo and three attributes to
     *  merge in.
     * 
     * @param oldExtenderInfo The existing extender info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     */        
    public ExtenderInfo(ExtenderInfo oldExtenderInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3) {}
    
    /**
     * This is a shortcut constructor that takes an existing ExtenderInfo and four attributes to
     *  merge in.
     * 
     * @param oldExtenderInfo The existing extender info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     * @param a4 A new attribute.
     */            
    public ExtenderInfo(ExtenderInfo oldExtenderInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3, MemberAttribute a4) {}
    
    /**
     * Determines whether the value of the extender property can be reset.  If there
     * is a DefaultValueAttribute, then this will return true if getValue returns
     * something different than the default value.  If there is a reset method and
     * a shouldPersist method, this will return what shouldPersist returns.
     * If there is just a reset method, this always returns true.  If none of these
     * cases apply, this returns false.
     *
     * @param extender The provider who stores the property value.
     * @param extendee The object who the property is being proffered to.
     */
    public boolean canResetValue(Object extender, Object extendee) {}
    
    /**
     * Determines whether the value of the extender property should be persisted.  In
     * other words, it indicates whether the state of the extender is distinct
     * from when the component is first instantiated. If there is a default
     * value specified in this ExtenderInfo, it will be compared against the
     * property's current value to determine this.  If there is't, the
     * shouldPersistXXX method is looked for and invoked if found.  If both
     * these routes fail, true will be returned.  
     * <p>
     * If this returns false, a tool should not persist this property's value.
     * 
     * @param extender The provider who stores the property value.
     * @param extendee The object who the property is being proffered to.
     * 
     * @return whether the property should be persisted for this 
     * extender-extendee pair.
     */
    public boolean shouldPersistValue(Object extender, Object extendee) {}
    
    /**
     * Determines whether the extender property is read-only.
     * 
     * @return whether this property is read only.
     */
    public boolean getIsReadOnly() {}

    /**
     * Retrieves the shouldPersist<ExtenderName> method of the extender
     * component. 
     * 
     * @return the shouldPersistXXX method pointer.
     */
    public Method getShouldPersistMethod() {}
    
    /**
     * Retrieves the get<ExtenderName> method of the extender component. 
     * 
     * @return the getXXX method pointer.
     */
    public Method getGetMethod() {}
    
    /**
     * Defines an extender property of a component. 
     * 
     * @return type of extendee.
     */
    public Class getExtendeeType() {}
    
    /**
     * Defines an extender property of a component. 
     * 
     * @return type of property.
     */
    public Class getType() {}
    
    /**
     * Retrieves the reset<ExtenderName> method of the extender component.
     * 
     * 
     * @return the resetXXX method pointer.
     */
    public Method getResetMethod() {}
    
    /**
     * Retrieves the set<ExtenderName> method of the extender component. 
     * 
     * @return the setXXX method pointer.
     */
    public Method getSetMethod() {}
    
    /**
     * Retrieves the current value of the extender property for the specified
     * extender-extendee pair.   An 
     * exception in the getXXX method will pass through.
     * 
     * @param extender The provider who stores the property value.
     * @param extendee The object who the property is being proffered to.
     * 
     * @return the value of the property for the given extender-extendee pair.
     */
    public Object getValue(Object extender, Object extendee) {}
    
    /** 
     * Retrieves the value editor of the extender property.   If a value editor is passed
     * in as a ValueEditorAttribute, that value editor will be instantiated.
     * If no such attribute was found, a system value editor will be looked for.
     * See IValueEditor for a description of how system value editors are found.
     * If there is no system value editor, null is returned.
     * 
     * @return the value editor for the property.
     * 
     * @see IValueEditor
     */
    public IValueEditor getValueEditor() {}
    
    /** 
     * Retrieves the value editor of the extender property.   If a value editor is passed
     * in as a ValueEditorAttribute, that value editor will be instantiated.
     * If no such attribute was found, a system value editor will be looked for.
     * See IValueEditor for a description of how system value editors are found.
     * If there is no system value editor, null is returned.  If the value editor found
     * takes an IEditorSite in its constructor, the parameter will be passed in. 
     * 
     * @param the site.
     *  
     * @return the value editor for the property.
     * 
     * @see IValueEditor
     */
    public IValueEditor getValueEditor(IEditorSite site) {}
    
    /**
     * Resets the value of the extender property for the specified extender-extendee pair.  If there was a default value passed 
     * in as a DefaultValueAttribute, that value will be set as the value of 
     * the property on the component.  If there was no default value passed 
     * in, a resetXXX method will be looked for.  If one is found, it will be 
     * invoked.  If one is not found, this is a nop.
     * 
     * @param extender The provider who stores the property value.
     * @param extendee The object who the property is being proffered to.
     */
    public void resetValue(Object extender, Object extendee) {}
    
    /**
     * Sets the value of the extender property for the specified extender-extendee pair.   If the value specified is invalid,
     * the component should throw an exception which will be passed up.  The 
     * component designer should design the property so that getXXX following 
     * a setXXX should return the value passed in if no exception was thrown 
     * in the setXXX call.
     * 
     * @param extender The provider who stores the property value.
     * @param extendee The object who the property is being proffered to.
     * @param value The new value for the property on the component.
     */
    public void setValue(Object extender, Object extendee, Object value) {}
}