//STRIPPEDSOURCE//
package com.ms.wfc.core;

import com.ms.wfc.ui.*;

/**
 * The IClassInfo interface is used to provide meta-information for a
 * component that cannot be determined from the component itself. Examples of
 * such meta-information include a component's set of properties, events, it's bitmap,
 * and so on.
 * <p>
 * A component never implements the IClassInfo interface directly. Rather,
 * the component implements a public static inner class named "ClassInfo"
 * which in turn implements the IClassInfo interface (usually extending the 
 * ClassInfo of the superclass).
 * <p>
 * This method of implementation allows the meta-information to be separated 
 * from the class itself in situations where the meta-information is not required
 * The example below shows a base class called Window and its associated ClassInfo 
 * implementation:
 * <pre>
 *   public class Window
 *   {
 *     // Members of class Window
 *
 *     public static class ClassInfo implements IClassInfo
 *     {
 *       // Implementations of all methods of the IClassInfo interface
 *     }
 *   }
 * </pre>
 * For a component class that is a descendant of another component class, the
 * ClassInfo of the descendant component class typically extends the ClassInfo
 * of the base component class. For example:
 * <pre>
 *   public class Button extends Window
 *   {
 *     // Members of class Button
 *
 *     public static class ClassInfo extends Window.ClassInfo
 *     {
 *       // Overrides of methods of the IClassInfo interface
 *     }
 *   }
 * </pre>
 * Even though a descendant component's ClassInfo will typically inherit from
 * the base component's ClassInfo, it isn't required to do so. Indeed, a
 * descendant component's ClassInfo may be implemented from scratch, or may
 * inherit from a ClassInfo higher up in the component hierarchy. This is
 * useful in situations where a component wants to omit properties that were
 * introduced by a base class.
 * <p>
 * Compilation of a component class with an inner ClassInfo class produces two
 * class files that follow the naming pattern XXX.class and XXX$ClassInfo.class,
 * where XXX is the name of the component class. The WFC component manager
 * uses this pattern to identify classes that support meta-information.
 *
 * @security(checkClassLinking=on)
 */
public interface IClassInfo {
    /**
     * Specifies the attributes of the component class.   An example of a
     * ClassInfo attribute is the ShowInToolboxAttribute, which indicates
     * whether this component should be shown in a tool's toolbox.
     *
     * @param attributes    An IAttributes interface that will function as
     *                      a callback for the attribute list.
     *
     * @see IAttributes
     * @see ShowInToolboxAttribute
     */
    public void getAttributes(IAttributes attributes);

    /**
     * Retrieves the customizer of the specified component.  The component parameter is
     * guaranteed to be an instance of the ClassInfo's associated component
     * class (or a subclass thereof). The return value may be null if the
     * component doesn't support a customizer. 
     * 
     * @param component The component to get the customizer for.
     * 
     * @return The customizer for component.
     * @see ICustomizer
     */
    public ICustomizer getCustomizer(Object component);

    /**
     * Retrieves the name of the default event of the component class.  The exact implications of being a default event are defined by
     * the tool that is editing the component, but it could for example mean that
     * the handler method of the event is shown in the code window
     * when a user double-clicks the component in the design environment.
     * 
     * @return The name of the default event.
     */
    public String getDefaultEventName();

    /**
     * Retrieves the name of the default property of the component class.  The exact implications of being a default property are defined by
     * the tool that is editing the component, but it could for example mean that
     * when the component's properties are examined in the Properties window, the
     * default property is initially selected.
     * 
     * @return The name of the default property.
     */
    public String getDefaultPropertyName();

    /**
     * Specifies the events of the component class.  The procedure for adding events
     * is very similar to the procedure for properties.
     *
     * @param events    An IEvents interface that will function as a callback
     *                  for the event list.
     *
     * @see IEvents
     * @see EventInfo
     */
    public void getEvents(IEvents events);

    /**
     * Specifies the extender properties of the component class.  The procedure for adding 
     * extenders is similar to the procedure for properties.
     *
     * @param extenders An IExtenders interface that will function as a callback
     *                  for the extender list.
     *
     * @see IExtenders
     * @see ExtenderInfo
     * @see IExtenderProvider
     */
    public void getExtenders(IExtenders extenders);

    /**
     * Specifies the properties of the component class. 
     * For a component class that is a descendant of another component class, 
     * the getProperties() method will typically call super.getProperties() 
     * before adding the properties that are particular to the descendant 
     * class. An example of an implementation of the getProperties() method 
     * follows below.
     * <pre>
     *   public class Edit extends Control
     *   {
     *     // Members of class Edit
     *
     *     public static class ClassInfo extends Control.ClassInfo
     *     {
     *         public static final PropertyInfo readOnly = 
     *	           new PropertyInfo(Edit.class, "readOnly", boolean.class,
     *	               DefaultValueAttribute.FALSE,
     *                 new DescriptionAttribute("Indicates whether the edit is read-only."));
     * 
     *	       public static final PropertyInfo maxLength =
     *             new PropertyInfo(Edit.class, "maxLength", int.class,
     *                 new DefaultValueAttribute(new Integer(10)),
     *	               new DescriptionAttribute("Indicates the maximum length in the edit control."));
     *
     *       public static final PropertyInfo backColor =
     *			new PropertyInfo(Control.ClassInfo.backColor,
     *				DefaultValueAttribute.COLOR_WINDOW);
     * 
     *       public getProperties(IProperties properties)
     *       {
     *         super.getProperties(properties);
     *         properties.add(readOnly);
     *         properties.add(maxLength);
     *	       properties.add(backColor);
     *       }
     *     }
     *   }
     * </pre>
     * UNDONE: add a browsable example too. giodl <p>
     * 
     * Each call to properties.add() defines a new property or masks an     
     * already defined property of the same name with new attributes. A call 
     * to add() takes a PropertyInfo, which encapsulates the information about
     * the property. 
     *
     * @param props An IProperties interface that will function as a callback
     *              for the property list.
     *
     * @see IProperties
     * @see PropertyInfo
     *
     */
    public void getProperties(IProperties props);

    /**
     * Retrieves the image that is used to represent the component in toolboxes, list views,
     * and so on.   If you ultimately extend com.ms.wfc.core.ClassInfo,
     * the default implementation will look for Foo.bmp in the same directory
     * as Foo.class and use that.  The bottom left pixel is the color that is
     * used for transparency purposes.
     *
     * @return A Bitmap object that contains an iconic representation of the
     *         component.  May return NULL if no such bitmap exists.
     *
     */
    public Bitmap getToolboxBitmap();
}