//STRIPPEDSOURCE//
package com.ms.wfc.core;

/**
 * The IComponent interface represents a component. To be a component, a class
 * must implement IComponent. Components can be hosted in containers. For each
 * component in a container, the container creates a site that acts as a
 * binding link between the component and the container.
 *
 * @security(checkClassLinking=on)
 */
public interface IComponent
{
    /**
     * Assigns a delegate to the dispose event. The OnDispose event is fired when the
     * component's dispose() method is called. Components that maintain
     * references to other components may want to register OnDispose event
     * handlers for the referenced components such that the references can be
     * cleared when the components are disposed.
     */
    public void addOnDispose(EventHandler value);

    /**
     * Disposes of the component.  A call to dispose() indicates that the user of the
     * component has no further need for the component. An implementation of
     * dispose() must:
     *
     * (1) Remove any references the component is holding to other components.
     *     This is typically accomplished by assigning null to any fields that
     *     contain references to other components.
     *
     * (2) Release any system resources that are associated with the component,
     *     such as file handles, window handles, or database connections.
     *
     * (3) Dispose any child components, such as child windows or sub-menu items,
     *     by calling the dispose() method of those components. The isChildOf()
     *     method of the components that are disposed in this fashion must return
     *     return true when passed this component.
     *
     * (4) Remove the component from its container. If the component has a site,
     *     it must execute "site.getContainer().removeComponent(this)", where
     *     "site" is the current (non-null) site.
     *
     * (5) Fire an OnDispose event to any event handlers that have been
     *     registered through the addOnDispose() method.
     *
     * Ideally, a call to dispose() will revert a component to the state it was
     * in immediately after it was created. This is however not a requirement,
     * and following a call to dispose(), a component is permitted to raise
     * exceptions for operations that cannot meaningfully be performed.
     */
    public void dispose();

    /**
     * Retrieves the site of the component.  The value returned must be the value
     * that was passed to the most recent call to setComponentSite(), or null 
     * if no calls have been made to setComponentSite().
     */
    public IComponentSite getComponentSite();

    /**
     * Determines whether this component is a child of the specified component.  A component is
     * considered a child of a given parent component if the parent component's
     * implementation of the dispose() method also disposes the child component.
     * Examples of child components are child windows and sub-menus.
     */
    public boolean getChildOf(IComponent component);

    /**
     * Removes the delegate that is assigned to the dispose event.
     */
    public void removeOnDispose(EventHandler value);

    /**
     * Sets the site of the component.  A non-null value indicates that the
     * component has been added to a container, and a null value indicates that
     * the component is being removed from a container. An implementation of
     * setComponentSite() is required to store the given value and return the 
     * same value from subsequent calls to getComponentSite().
     */
    public void setComponentSite(IComponentSite value);
}