//STRIPPEDSOURCE//
package com.ms.wfc.core;



import com.ms.wfc.ui.*;
import com.ms.wfc.win32.*;
import com.ms.win32.*;

/**
 * This interface defines design page, which is a page of a dialog that allows for 
 * changing design time settings.  A form package typically shows this in one
 * tab of a tabbed dialog, but that is only one possible implementation.  There
 * are two important interfaces to consider here:
 *
 * @see IDesignPageSite
 *
 *      There is one of these for each design page.  This provides a window for
 *      the design page to parent itself in.  Also, the page will notify that
 *      it is dirty through this interface.  @see DesignForm is a stock implementation
 *      of IDesignPageSite.
 *
 * IDesignPage
 *
 *      This is the page itself.  This typically displays UI for setting 
 *      a component's design time properties that cannot easily be set through 
 *      a typical property sheet.  A page may be asked to size itself
 *      larger than it's default size to unify the look of multiple pages
 *      in the same dialog, but will never be asked to become smaller than
 *      its default.  Pages should support dynamic resizing, but it is not
 *      a requirement.  @see DesignPage is a stock implementation of
 *      IDesignPage.
 *
 * @security(checkClassLinking=on)
 */
public interface IDesignPage {

    /**
     * Activates and displays the page.
     */
    public void activate();

    /**
     * Applies changes to all the objects in this page's object list.
     */
    public void apply();

    /**
     * Deactivates the page.  A page may be deactivated and later reactivated.
     */
    public void deactivate();

    /**
     * Retrieves the control that represents the window for this page.
     *
     * @return a Control object.
     */
    public Control getControl();

    /**
     * Retrieves the width and height of this page.  The caller may use this
     * to size it's UI appropriately.
     *
     * @return the size of the page.
     */
    public Point getSize();

    /**
     * Retrieves the title of the design page.  The caller may use this to show
     * the name of this page before it's actually activated.
     *
     * @return the title of page.
     */
    public String getTitle();

    /**
     * Gives the design page a chance to process messages before the caller
     * uses them.
     *
     * @param   <i>msg</i> @see com.ms.wfc.win32.MSG to process.
     * @return  <b>true</b> if this page processed the message, or <b>false</b>
     *          otherwise. If this returns true, the message will be eaten.
     *          Otherwise, the caller will continue to process the message.
     */
    public boolean getPageMessage(MSG msg);

    /**
     * Called before activation to setup the array of objects that this page is
     * modifying properties for.  The objects are not guarenteed to be of any
     * particular type; their only commonality is that they have all specified 
     * that they use this design page class.
     *
     * @param <i>objects</i> The array of objects this page is to modify.
     */
    public void setObjects(Object [] objects);

    /**
     * Called before this page is activated to setup the site we are to use
     * to notify the caller of changes.
     *
     * @param <i>site</i> The @see IDesignPageSite to which this page is attached.
     */
    public void setSite(IDesignPageSite site);

    /**
     * Called to enlarge the size of this page.  This is done after the 
     * caller has determined the sizes of all pages.  If this page
     * supports resizing to a larger size it should do so.  A design page will
     * never be requested to size smaller than it's default size.
     *
     * @param <i>size</i> The new size.
     */
    public void setSize(Point size);

    /**
     * Called to determine whether or not the help icon should be shown at the
     * top of the dialog.  If one or more pages supports the icon, then it will
     * appear.  It will be greyed out for pages that do not support help.
     *
     * @return <b>true</b> to enable the help icon for this page, or <b>false</b>
     *         to disable it.
     */
    public boolean supportsHelp();
}