//STRIPPEDSOURCE//

package com.ms.wfc.core;

import com.ms.wfc.app.Locale;

/**
 * A resource manager is responsible for managing multiple resource sets. The
 * resource sets are loaded on demand. Each property should cascade, in that
 * if a specific resource set doesn't contain that property, then the parent
 * resource set value should be returned.
 *
 * @security(checkClassLinking=on)
 */
public interface IResourceManager {

    /**
     * Determines whether the specified property is stored for a given locale. 
     * 
     * @param name Name of the property to retrieve.
     *
     * @return Property value.
     */
    public boolean containsProperty(String name);

    /**
     * Determines whether the specified property is stored for a given locale. 
     * 
     * @param locale Locale to check
     * @param name Name of the property to retrieve.
     *
     * @return Property value.
     */
    public boolean containsProperty(Locale locale, String name);

    /**
     * Retrieves a string for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>null</b> if it isn't set.
     */
    public String getString(String name);
    
    /**
     * Retrieves a string for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>null</b> if it isn't set.
     */
    public String getString(Locale locale, String name);

    /**
     * Retrieves a boolean value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>false</b> if it isn't set.
     */
    public boolean getBoolean(String name);
    
    /**
     * Retrieves a boolean value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>false</b> if it isn't set.
     */
    public boolean getBoolean(Locale locale, String name);

    /**
     * Retrieves a character for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public char getChar(String name);
    
    /**
     * Retrieves a character for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public char getChar(Locale locale, String name);

    /**
     * Retrieves a byte value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public byte getByte(String name);
    
    /**
     * Retrieves a byte value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public byte getByte(Locale locale, String name);

    /**
     * Retrieves a short value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public short getShort(String name);
    
    /**
     * Retrieves a short value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public short getShort(Locale locale, String name);

    /**
     * Retrieves an integer for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public int getInt(String name);
    
    /**
     * Retrieves an integer for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public int getInt(Locale locale, String name);

    /**
     * Retrieves a long value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public long getLong(String name);
    
    /**
     * Retrieves a long value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0 if it isn't set.
     */
    public long getLong(Locale locale, String name);

    /**
     * Retrieves a float value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0.0 if it isn't set.
     */
    public float getFloat(String name);
    
    /**
     * Retrieves a float value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0.0 if it isn't set.
     */
    public float getFloat(Locale locale, String name);

    /**
     * Retrieves a double value for the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0.0 if it isn't set.
     */
    public double getDouble(String name);
    
    /**
     * Retrieves a double value for the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or 0.0 if it isn't set.
     */
    public double getDouble(Locale locale, String name);

    /**
     * Retrieves the value of the specified property. 
     * 
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>null</b> if it isn't set.
     */
    public Object getObject(String name);
    
    /**
     * Retrieves the value of the specified property. 
     * 
     * @param locale Locale to retrieve the property for
     * @param name Name of the property to retrieve.
     * 
     * @return the property value, or <b>null</b> if it isn't set.
     */
    public Object getObject(Locale locale, String name);

    /**
     * Retrieves the names of all properties for a given locale. 
     * 
     * @return array containing all the names stored for the current locale.
     */
    public String[] getNames();

    /**
     * Retrieves the names of all properties for a given locale. 
     * 
     * @param locale Locale to retrieve the names for
     * 
     * @return array containing all the names stored for the specified locale.
     */
    public String[] getNames(Locale locale);

    /**
     * Retrieves the values of all properties for a given locale. 
     * 
     * @return array containing all the values stored for the current locale.
     */
    public Object[] getProperties();

    /**
     * Retrieves the values of all properties for a given locale. 
     * 
     * @param locale Locale to retrieve the names for
     * 
     * @return array containing all the values stored for the specified locale.
     */
    public Object[] getProperties(Locale locale);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, String value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, String value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, boolean value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, boolean value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, char value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, char value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, byte value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, byte value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, short value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, short value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, int value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, int value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, long value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, long value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, float value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, float value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, double value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, double value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(String name, Object value);

    /**
     * Sets the value of the specified property. 
     * 
     * @param locale Locale to set the property for.
     * @param name Name of the property to set the value for.
     * @param value Value to set the property to.
     */
    public void setProperty(Locale locale, String name, Object value);

    /**
     * Retrieves the resource set that is storing the properties for the specified locale. 
     * 
     * @param locale Locale to retrieve the resource set for.
     * 
     * @return the resource set that is storing the property for the specified
     *         locale.
     */
    public IResourceSet getResourceSet(Locale locale);
    
    /**
     * Sets the resource set that will store the properties for the specified locale. 
     * 
     * @param locale Locale that <i>set</i> will store the properties for.
     * @param set Resource set that will store the properties.
     */
    public void setResourceSet(Locale locale, IResourceSet set);

    /**
     * Saves all properties for all locales to disk.  The files will be
     * stored in <i>directory</i> and will all have the base name of
     * <i>baseName</i>. <i>baseName</i> should not contain any extension.
     * <p>
     * Each locale's resource set will be saved with a unique name, based
     * on the language and default country name of that locale.
     * 
     * @param directory Directory to store the files in.
     * @param baseName base name to give to all the files.
     */
    public void saveAllProperties(String directory, String baseName);
}