//STRIPPEDSOURCE//

package com.ms.wfc.core;

import com.ms.wfc.ui.*;

/**
 * The IValueEditor interface represents the implementation of an editor for
 * values of a given type. The primary use of value editors is to edit property
 * values in a property inspector. A class provides a value editor by
 * implementing a public static inner class named "Editor" which in turn
 * implements the IValueEditor interface. This method of implementation allows
 * the value editor to be separated from the class itself in situations where
 * the value editor is not required (such as when the class is deployed). The
 * example below shows a class called Color and its associated value editor:
 *
 *   public class Color
 *   {
 *     // Members of class Color
 *
 *     public static class Editor implements IValueEditor
 *     {
 *       // Implementations of IValueEditor methods
 *     }
 *   }
 *
 * When a class implements a value editor, the value editor implementation is
 * automatically inherited by subclasses that don't implement their own value
 * editors.
 *
 * To create an instance of a value editor, the component manager first looks
 * for a constructor that takes a single argument of type java.lang.Class. If
 * such a constructor exists, it will be called with the actual Class object
 * for which the value editor is being created. Otherwise the component manager
 * will call the parameterless constructor of the value editor class.
 *
 * If a value editor requires special knowledge about the component who's property
 * is being edited using the value editor, it can have a constructor that takes
 * an IEditorSite object.  This takes precedence over either of the two constructors
 * mentioned above.  This value editor will be created for each instance of use,
 * as opposed to the value editors above which are reused.
 *
 * The ComponentManager provides the ability to register value editors for a
 * particular type through the registerValueEditorClass() method. This is
 * useful for value editors that aren't implemented as inner classes.
 *
 * The cor.design.ValueEditor class provides a default implementation of the
 * IValueEditor interface. Value editors may choose to inherit from ValueEditor
 * instead of implementing the entire IValueEditor interface.
 * 
 *
 * @security(checkClassLinking=on)
 */
public interface IValueEditor { 
    /**
     * Specifies that the value editor should not accept arbitrary text input. Indicates that the value editor does not
     * support accepting arbitrary text in the getValueFromText() method. It should
     * accept text that is returned from getTextFromValue(). This style will typically 
     * cause a property editor to disallow typing in the edit box for properties 
     * of the* value editor's type.
     */
    public static final int STYLE_NOEDITABLETEXT = 0x00000001;

    /**
     * Specifies that the property browser should paint a representation of the property. Indicates that the value editor implements
     * the paintValue() method. This style will typically cause a property editor
     * to paint a representation of properties of the value editor's type.
     */
    public static final int STYLE_PAINTVALUE = 0x00000002;

    /**
     * Specifies that a property browser should display a drop-down list for the property. Indicates that the value editor implements
     * the getValues() method. This style will typically cause a property editor
     * to display a drop-down list for properties of the value editor's type.
     */
    public static final int STYLE_VALUES = 0x00000004;

    /**
     * Specifies that the property browser should display an ellipsis button for the property. Indicates that the value editor implements
     * the editValue() method. This style will typically cause a property editor
     * to display an ellipsis button for properties of the value editor's type.
     */
    public static final int STYLE_EDITVALUE = 0x00000008;

    /**
     * Specifies that the property browser should display a drop-down arrow for the property.  Indicates that if STYLE_EDITVALUE is returned, 
     * a drop down arrow should be shown instead of an ellipsis button.  This is useful
     * if the value editor plans to show a drop down in editValue as opposed to launching
     * a modal dialog.
     */
    public static final int STYLE_DROPDOWNARROW = 0x00000010;

    /**
     * Specifies that changes made to the property should be displayed immediately. Indicates that values of this type are
     * suitable for updating even when the new value is incomplete. This style
     * will typically cause a property editor to update the underlying property
     * after each change made in the editor (e.g. as each character is typed in
     * an edit box).
     */
    public static final int STYLE_IMMEDIATE = 0x00000020;

    /**
     * Specifies that the property browser should show the public static final
     * fields of the class being edited.  Indicates that a browser should show
     * the fields of the value being edited.
     */
    public static final int STYLE_SHOWFIELDS = 0x00000040;

    /**
     * Specifies that a property can have subproperties that can be edited. Indicates that values of this type can have
     * "sub properties" that can be edited.
     */
    public static final int STYLE_PROPERTIES = 0x00000080;


    /**
     * Specifies that arrays should not be expanded in a property browser.  Indicates that if the value is an
     * array, a property inspector should not drill down into the array.
     */
    public static final int STYLE_NOARRAYEXPANSION = 0x00010000;

    /**
     * Specifies that the property browser should not display an entry for the multiselection
     * of array elements.  Indicates that if the value is an
     * array, a property inspector should not have an entry that multi-selects
     * the elements of the array.
     */
    public static final int STYLE_NOARRAYMULTISELECT = 0x00020000;


    /**
     * Provides a user interface for editing the property value.getValue() and modifies or replaces
     * the value, passing the result into valueAccess.setValue().  This is where
     * an editor can launch a modal dialog or create a drop down editor to allow
     * the user to modify the value.  Host assistance in presenting UI to the user
     * can be found through the valueAccess.getService function.
     * 
     * @param valueAccess The way that editValue accesses and modifies a given value.
     * 
     * @see IValueAccess
     */
    public void editValue(IValueAccess valueAccess);

    /**
     * Retrieves the fully qualified name of a constant that represents the specified value. The returned value
     * must consist of a fully qualified class name followed by a period followed
     * by the identifier of a public static final field of the given class, for
     * example "com.ms.wfc.Color.BLACK". This method is typically used by a
     * design-time environment that generates source code to initialize a
     * component's properties.
     */
    public String getConstantName(Object value);

    /**
     * Retrieves the subproperties of the specified value.   These sub properties
     * represent values that together compose the given value.  Each of these values
     * can be edited by the user, and getValueFromSubPropertyValues() will be called to
     * see if a new set of values can compose a new value of this type. This
     * method will only be called if the STYLE_PROPERTIES style is present.
     * 
     * @param value Value to be decomposed
     * 
     * @return subproperties The decomposition of the value into editable values.
     * 
     * @see getValueFromSubPropertyValues
     */
    public SubProperty[] getSubProperties(Object value);

    /**
     * Retrieves the style flags for the value editor.  The returned value must be
     * a combination of the STYLE_XXX bit masks defined in this interface.
     */
    public int getStyle();

    /**
     * Converts the specified value to a text string.  The return value may be null
     * if the value cannot be converted to a text string.
     */
    public String getTextFromValue(Object value);

    /**
     * Retrieves the common values for the editor's type.  In order to
     * display these values in a drop-down list, a property editor will typically
     * call getTextFromValue() for each of the values in the list.
     */
    public Object[] getValues();

    /**
     * Creates a value of this editor's type from the specified values.  If it is successful, it returns the composed value, otherwise
     * it returns null. This array of values is usually a modified version of
     * the values contained in the sub properties returned by getSubProperties().
     * This method will only be called if the STYLE_PROPERTIES style is present.
     *
     * @param values array of values.
     * 
     * @return value that results from the composition of the inputs. (null if a
     * composition wasn't possible.)
     * 
     * @see getSubProperties 
     */
    public Object getValueFromSubPropertyValues(Object[] values);

    /**
     * Converts the specified text string to a value.  If the editor specifies the 
     * STYLE_NOEDITABLETEXT style, this function will only be called with
     * strings that were previously returned by getTextFromValue.
     */
    public Object getValueFromText(String text);

    /**
     * Paints a representation of the specified value.  This method will only be called if the value editor
     * specifies the STYLE_PAINTVALUE style.
     */
    public void paintValue(Object value, Graphics g, Rectangle rect);
}