//STRIPPEDSOURCE//

package com.ms.wfc.core;

import com.ms.wfc.util.*;
import java.lang.reflect.*;

/**
 * Defines a property of a component.&nbsp;  Properties are the main way that a user can 
 * set up the state of a component. 
 * <p>
 * The PropertyInfo class takes a component class that the property lives on,
 * a property name, the type of the property, and various attributes for the
 * property.
 * <p>
 * For a property named XXX of type YYY, the associated component class is 
 * required to implement two methods of the following 
 * form:
 * <pre>
 *   public YYY getXXX();
 *   public void setXXX(YYY value);
 * </pre>
 * The component class can optionally implement two additional methods of 
 * the following form:
 * <pre>
 *   public boolean shouldPersistXXX();
 *   public void resetXXX();
 * </pre>
 * These methods deal with a property's default value. The shouldPersistXXX()
 * method returns true if the current value of the XXX property is different
 * than it's default value, so that it should be persisted out.  The resetXXX() 
 * method resets the XXX property to its default value. If the PropertyInfo
 * includes the default value of the property (using the DefaultValueAttribute),
 * the shouldPersistXXX() and resetXXX() methods are ignored. 
 * <p>
 * If the PropertyInfo includes a reference to a value editor
 * (the ValueEditorAttribute), then that value editor will be used to 
 * edit the property. Otherwise, a system-provided value editor will be used.
 * <p>
 * Various attributes can be passed to the PropertyInfo, as are described in
 * MemberAttribute.
 * <p>
 * PropertyInfos can be obtained by a user programmatically through the
 * ComponentManager.
 * 
 * @see IClassInfo
 * @see MemberInfo
 * @see MemberAttribute
 * @see DefaultValueAttribute
 * @see ValueEditorAttribute
 * @see ComponentManager
 *
 * @security(checkClassLinking=on)
 */
public class PropertyInfo extends MemberInfo {
    
    static final Class[] argsNone = new Class[0];
    static final Object noDefault = new Object();
    
    private Class type;
    private Object valueEditorInfo;
    private Object defaultValue;
    private Method getMethod;
    private Method setMethod;
    private Method shouldPersistMethod;
    private Method resetMethod;
    
    /**
     * The main constructor for PropertyInfos. 
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param attributes The attributes of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute[] attributes) {}
    
    /**
     * A constructor for PropertyInfos that have no attributes. 
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type) {}
    
    /**
     * A helfpul constructor for PropertyInfos with one attribute.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1) {}
    
    /**
     * A helfpul constructor for PropertyInfos with two attributes.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2) {}
    
    /**
     * A helfpul constructor for PropertyInfos with three attributes.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2, MemberAttribute a3) {}
    
    /**
     * A helfpul constructor for PropertyInfos with four attributes.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     * @param a4 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2,
        MemberAttribute a3, MemberAttribute a4) {}
    
    /**
     * A helfpul constructor for PropertyInfos with five attributes.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     * @param a4 An attribute of the property.
     * @param a5 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2,
        MemberAttribute a3, MemberAttribute a4,
        MemberAttribute a5) {}
    
    /**
     * A helfpul constructor for PropertyInfos with six attributes.
     * 
     * @param componentClass The type of the component the property lives on.
     * @param name The name of the property.
     * @param type The type of the property.
     * @param a1 An attribute of the property.
     * @param a2 An attribute of the property.
     * @param a3 An attribute of the property.
     * @param a4 An attribute of the property.
     * @param a5 An attribute of the property.
     * @param a6 An attribute of the property.
     */
    public PropertyInfo(Class componentClass, String name, Class type,
        MemberAttribute a1, MemberAttribute a2,
        MemberAttribute a3, MemberAttribute a4,
        MemberAttribute a5, MemberAttribute a6) {}
    
    /**
     * This constructor takes an existing PropertyInfo and modifies it by merging in the
     * passed-in attributes.
     * 
     * @param oldPropertyInfo The existing property info.
     * @param attributes The new attributes.
     */
    public PropertyInfo(PropertyInfo oldPropertyInfo, MemberAttribute[] attributes) {}
    
    /**
     * This is a shortcut constructor that takes an existing PropertyInfo and one attribute to
     *  merge in.
     * 
     * @param oldPropertyInfo The existing property info.
     * @param a1 A new attribute.
     */
    public PropertyInfo(PropertyInfo oldPropertyInfo, MemberAttribute a1) {}
    
    /**
     * This is a shortcut constructor that takes an existing PropertyInfo and two attributes to
     *  merge in.
     * 
     * @param oldPropertyInfo The existing property info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     */    
    public PropertyInfo(PropertyInfo oldPropertyInfo, MemberAttribute a1,
        MemberAttribute a2) {}
    
    /**
     * This is a shortcut constructor that takes an existing PropertyInfo and three attributes to
     *  merge in.
     * 
     * @param oldPropertyInfo The existing property info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     */        
    public PropertyInfo(PropertyInfo oldPropertyInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3) {}
    
    /**
     * This is a shortcut constructor that takes an existing PropertyInfo and four attributes to
     *  merge in.
     * 
     * @param oldPropertyInfo The existing property info.
     * @param a1 A new attribute.
     * @param a2 A new attribute.
     * @param a3 A new attribute.
     * @param a4 A new attribute.
     */            
    public PropertyInfo(PropertyInfo oldPropertyInfo, MemberAttribute a1,
        MemberAttribute a2, MemberAttribute a3, MemberAttribute a4) {}    
    
    /**
     * Determines whether the value of the property can be reset.  If there
     * is a DefaultValueAttribute, then this will return true if getValue returns
     * something different than the default value.  If there is a reset method and
     * a shouldPersist method, this will return what shouldPersist returns.
     * If there is just a reset method, this always returns true.  If none of these
     * cases apply, this returns false.
     *
     * @param component The component who's property's value we should test for resetability.
     */
    public boolean canResetValue(Object component) {}

    /**
     * Determines whether the property is read-only.
     * 
     * @return whether this property is read only.
     */
    public boolean getIsReadOnly() {}

    /**
     * Retrieves the shouldPersist<PropertyName> method of the component
     * class. 
     * 
     * @return the shouldPersistXXX method pointer.
     */
    public Method getShouldPersistMethod() {}
    
    /**
     * Retrieves the get<PropertyName> method of the component class. 
     * 
     * @return the getXXX method pointer.
     */
    public Method getGetMethod() {}
    
    /**
     * Defines a property of a component.&nbsp; 
     * 
     * @return type of property.
     */
    public Class getType() {}
    
    /**
     * Retrieves the reset<PropertyName> method of the component class. 
     * 
     * @return the resetXXX method pointer.
     */
    public Method getResetMethod() {}
    
    /**
     * Retrieves the set<PropertyName> method of the component class. 
     * 
     * @return the setXXX method pointer.
     */
    public Method getSetMethod() {}
    
    /**
     * Retrieves the current value of the property.   An exception in the getXXX
     * method will pass through.
     * 
     * @param component The component who's property's value will be retrieved.
     * 
     * @return the value of the property for the given component.
     */
    public Object getValue(Object component) {}
    
    /** 
     * Retrieves the value editor of the property.   If a value editor is passed
     * in as a ValueEditorAttribute, that value editor will be instantiated.
     * If no such attribute was found, a system value editor will be looked for.
     * See IValueEditor for a description of how system value editors are found.
     * If there is no system value editor, null is returned.
     * 
     * @return the value editor for the property.
     * 
     * @see IValueEditor
     */
    public IValueEditor getValueEditor() {}
    
    /** 
     * Retrieves the value editor of the property.   If a value editor is passed
     * in as a ValueEditorAttribute, that value editor will be instantiated.
     * If no such attribute was found, a system value editor will be looked for.
     * See IValueEditor for a description of how system value editors are found.
     * If there is no system value editor, null is returned.  If the value editor found
     * takes an IEditorSite in its constructor, the parameter will be passed in. 
     * 
     * @param the site.
     *  
     * @return the value editor for the property.
     * 
     * @see IValueEditor
     */
    public synchronized IValueEditor getValueEditor(IEditorSite site) {}
    
    
    /**
     * Resets the value of the property.   If
     * there was a default value passed in as a DefaultValueAttribute, that 
     * value will be set as the value of the property on the component.  If
     * there was no default value passed in, a resetXXX method will be looked
     * for.  If one is found, it will be invoked.  If one is not found, this
     * is a nop.
     * 
     * @param component The component who's property's value should be reset.
     */
    public void resetValue(Object component) {}
    
    /**
     * Sets the value of the property.  If the
     * value specified is invalid, the component should throw an exception
     * which will be passed up.  The component designer should design the
     * property so that getXXX following a setXXX should return the value
     * passed in if no exception was thrown in the setXXX call.
     * 
     * @param component The component to set the property's value on.
     * @param value The new value for the property on the component.
     */
    public void setValue(Object component, Object value) {}
    
    /**
     * Determines whether the value of the property should be persisted.  In
     * other words, it indicates whether the state of the property is distinct
     * from when the component is first instantiated. If there is a default
     * value specified in this PropertyInfo, it will be compared against the
     * property's current value to determine this.  If there is't, the
     * shouldPersistXXX method is looked for and invoked if found.  If both
     * these routes fail, true will be returned.  
     * <p>
     * If this returns false, a tool should not persist this property's value.
     * 
     * @param component who's property should be examined for persistance.
     * 
     * @return whether the property should be persisted.
     */
    public boolean shouldPersistValue(Object component) {}

}