//STRIPPEDSOURCE//
package com.ms.wfc.html;
import com.ms.wfc.core.*;

import com.ms.wfc.util.HashTable;

/**
 * Represents a base class for Hypertext Markup Language (HTML) generation.   Call
 * DhElement.setHTMLGenrator with an instance of your override to enable.  On each tag, style
 * and attribute, the appropriate function is called and you will be given the opportunity to 
 * manipulate the element in any way you choose.
 * 
 * <p>Here is an example of a simple alternate generator that maps some CSS styles to format tags,
 *  and changes any "&lt;DIV&gt;" tags to "&lt;P&gt;" tags:
 *
 *  <PRE>
 *   public class MyHTMLGenerator {
 *                public final static STYLE_FONT_WEIGHT = 0;
 *      public final static STYLE_FONT_STYLE = 1;
 *      public final static TAG_DIV = 2;
 *       
 *              public MyHTMLGenerator(){
 *                      registerName( "font-weight", STYLE_FONT_WEIGHT );
 *                      registerName( "font-style", STYLE_FONT_STYLE );
 *                      registerName( "DIV", TAG_DIV );
 *              }
 *                      
 *              // we override this function to replace tags.
 *              public boolean onTagGenerate( int key, String strTagName ){
 *                      switch ( key ){  
 *                              case TAG_DIV:
 *                                      setTagName( "P" );
 *          // return false to cancel the default behavior
 *                                      return false;
 *                              break;
 *                      }
 *      // return true to proceed with the default behavior
 *                      return true;
 *              }
 *
 *   // we override this function to manipulate style output
 *   public boolean onStyleGenerate( int key, String styleName, String styleValue ){
 *      switch ( key ){
 *        case STYLE_FONT_WEIGHT:
 *           if ( styleValue.indexOf( "BOLD" ) != -1 ){
 *             addEnclosingTag( "B", null, true );
 *             return false;
 *           }
 *           break;
 *         case STYLE_FONT_STYLE:
 *           if ( styleValue.equalsIgnoreCase( "ITALIC" ) ){
 *             addEnclosingTag( "I", null, true );
 *             return false;
 *           }             
 *           break;
 *       }
 *       return true;
 *   }
 *              
 * }
 * </PRE>
 * 
 *
 * @security(checkClassLinking=on)
 */

public class DhHTMLGenerator {
    
    /**
     *  This value is returned by getNameKey if the name is not
     * registered. */
    public static final int     UNKNOWN_KEY = -1;

    
    
    

    private boolean m_supportsOM = true;
    private boolean m_supportsCSS = true;
    
    
    private StringBuffer sbTagName;
    private StringBuffer sbTagAttrs;
    private StringBuffer sbTagStyle;
    private StringBuffer sbBeforeTag;
    private StringBuffer sbBeforeContent;
    private StringBuffer sbAfterTag;
    private StringBuffer sbAfterContent;
    private StringBuffer sbContent;
    
    
    private char cStyleAssign       = ':';
    private char cStyleSeparator    = ';';
    private char cStyleQuote        = 0;
    private char cAttrAssign        = '=';
    private char cAttrQuote         = '\"';
    private char cAttrSeparator     = ' ';

    
    private DhNameValuePairs attrs;
    private DhNameValuePairs styles;


    
    private HashTable        m_keys = null;


    protected DhHTMLGenerator(){}

    
    
    
    
    
    
    
    
    

    /**
     * Invokes a callback with the default name for the current tag.   Perform any
     * desired operations within your override.
     *
     * Example of replacing <STRONG> with <BOLD>
     * <PRE>  
     * if ( strTagName.equalsIgnoreCase( "STRONG" ) ){
     *   setTagName( "BOLD" );
     *   return false;
     * }
     * </PRE>
     *
     * @param key The key for this tag, if registered. Otherwise -1
     * @param strTagName The default name for the current tag
     * @return Return <b>true</b> to use the default tag.
     * @see #setTagName
     * @see #getTagName
     */
    public boolean onTagGenerate( int key, String strTagName ){}

    public boolean onTagGenerate( String strTagName ){}



    /**
     * Invokes a callback for each attribute.   Override this function to perform any desired
     * additions or manipulations for the given attibibute.
     *
     * Example of moving WIDTH attribute into style text:
     * <PRE>  
     * if ( strName.equalsIgnoreCase( "WIDTH" ) ){
     *   addStyleString( "width", strValue );
     *   return false;
     * }
     * </PRE>
     *
     * @param strName the default name of the attribute
     * @param strValue the default value for the attribute
     * @return Return <b>true</b> to generate the default tag, <b>false</b> to skip it.
     * @see #addEnclosingTag
     * @see #addInnerTag
     * @see #addStyleString
     * @see #addAttrString
     */
    public boolean onAttrGenerate( String strName, String strValue ){}

    public boolean onAttrGenerate( int key, String strName, String strValue ){}

    /**
     * Invokes a callback for each style-attribute pair.   Override this function to perform any desired
     * additions or manipulations for the given style.<P>
     *
     * Example of non-CSS compatable code for font color:
     * <PRE>  
     * if ( strName.equalsIgnoreCase( "color" ) ){
     *   addEnclosingTag( "FONT", "COLOR=" + strValue );
     *   return false;
     * }
     * </PRE>
     * @param key The key for this style, if registered. Otherwise -1
     * @param strName the default name of the attribute
     * @param strValue the default value for the attribute
     * @return <b>true</b> to generate the default tag, or <b>false</b> to skip it.
     * @see #addEnclosingTag
     * @see #addInnerTag
     * @see #addStyleString
     * @see #addAttrString        
     */
    public boolean onStyleGenerate( int key, String strName, String strValue ){}

    public boolean onStyleGenerate( String strName, String strValue ){}
    
    /**
     * Specifies whether the target browser supports an object model.   The default is <b>false</b>.
     *
     * @param supportsOM <b>true</b> if the target supports the Document Object Model
     */
    public  final void setSupportsOM( boolean supportsOM ){}

    /**
     * Retrieves the target's ability to support the Document Object Model. 
     * @return <b>true</b> if the DOM is supported (currently MSIE 4.0 or greater only)
     */
    public final boolean getSupportsOM(){}

    /**
     * Specifies whether the target browser supports style sheets.   The default is <b>false</b>.
     *
     * @param supportsCSS <b>true</b> if the target supports the Document Object Model
     */        
    public final void setSupportsCSS( boolean supportsCSS ){}

    /**
     * Retrieves target's ability to support cascading style sheets. 
     * @return <b>true</b> if CSS is supported.
     */        
    public final boolean getSupportsCSS(){}

    /**
     * Generates browser-targeted HTML for the specified DhElement. generateHTML to 
     * generate the appropriate HTML from the supplied
     * tag, attributes, styles, and content
     * @internalonly
     */
    protected final StringBuffer generateHTML( StringBuffer sb, 
                                                            String strTag, 
                                                            boolean fClose, 
                                                            DhNameValuePairs attributes, 
                                                            DhNameValuePairs styles, 
                                                            StringBuffer sbContent ){}

    /**
     * Generates browser-targeted HTML for the specified DhElement. ms.wfc.html.DhElement.html">DhElement</a>.
     *
     * @param e The DhElement to generate code for based on this DhHTMLGenerator
     * @return A StringBuffer with the generated HTML.
     */
    public StringBuffer generateHTML( DhElement e ){}

    
    
    
    
    /**
     * Associates an integer value with a style or attribute name or tag.   If registered, you will receive this value
     * with the name in calls to onTagGenerate, onAttrGenerate, and onStyleGenerate.
     * Registering a key allows you to avoid string comparison in your handler functions.
     *
     * <P>NOTE: key cannot be -1;
     * <p>Example:
     * <PRE>
     * public class MyHTMLGenerator {
     *              public final static TAG_BR = 0;
     *      public final static TAG_DIV = 1;
     *       
     *              public MyHTMLGenerator(){
     *                      registerName( "BR", TAG_BR );
     *                      registerName( "DIV", TAG_DIV );
     *              }
     *                      
     *              // ...
     *              public boolean onTagGenerate( int key, String strTagName ){
     *                      switch ( key ){
     *                              case TAG_BR:
     *                              case TAG_DIV:
     *                                      setTagName( "P" );
     *                                      return false;
     *                              break;
     *                      }
     *                      return true;
     *              }
     *
     *              
     * }
     * </PRE>
     *
     * @param strName The tag, attribute name, or style name.
     * @param key     The key you wish to associate with that value.  Cannot be -1.
     * @return <b>true</b> if the name was successfully registered, 
     *         <b>false</b> if -1 is used or the key already is assigned to another name.
     */
    protected boolean registerName( String strName, int key ){}

    /**
     * Retrieves the integer key associated with the specified name.   This function is not case sensitive.
     *
     * @param strName The name of the tag, attribute, or style to retrieve a key for.
     * @return The key for the name, or UNKNOWN_KEY if it's not present.
     * @see #registerName
     * @see #UNKNOWN_KEY
     */
    protected int getNameKey( String strName ){}
    
    /**
     * Sets the delimiter items. 
     * @param cAttrAssign Assignement character for attributes (default is '=')
     * @param cAttrSeparator Seperator character between attributes (default is ' ')
     * @param cAttrQuote Delimiter to place around attribute values (default is '<b>"</b>')
     * @param cStyleAssign Assignement character for styles (default is ':')
     * @param cStyleSeparator Seperator character between (default is ';')
     * @param cStyleQuote Delimiter to place around style values (default is "'")
     */
    public final void setFormatItems( char cAttrAssign, char cAttrSeparator, char cAttrQuote, 
                                                   char cStyleAssign, char cStyleSeparator, char cStyleQuote ){}

    /**
     * Retrieves the value of a specified attribute. 
     * @param strAttrName The name of the attribute
     * @returns The value of the attriubute, or <b>null</b> if it is not present.
     */
    protected final String getAttributeValue( String strAttrName ){}

    /**
     * Retrieves the value of a specified style. 
     * @param strStyleName The name of the style
     * @returns The value of the style, or <b>null</b> if it is not present.
     */
    protected final String getStyleValue( String strStyleName ){}


    /**
     * Retrieves the current content string, which is the string between the HTML start and
     * end tags. 
     * @return The current content string.  
     */
    protected final String getContent(){}

    /**
     * Replaces the current content, destroying any existing content. 
     * @param strNewContent The new content string
     */
    protected final void setContent( String strNewContent ){}
    /**
     * Replaces the current tag name with the new value. 
     * @param strNewName The replacement tag name, can be <b>null</b> in which case the tag will not be generated, but it's content will.
     */
    protected final void setTagName( String strNewName ){}

    /**
     * Retreives the tag name for the current generation. 
     * @return The current tag name string.
     */
    protected final String getTagName(){}

    /**
     * Adds an enclosing tag around the current tag.   This tag will also enclose any existing
     * enclosing tags.
     * <P>
     * Example:<br>
     * 
     * <i><b>Original HTML:</b></i> &lt;SPAN id=span1&gt;Hello!&lt;/SPAN&gt;<br><br>
     * <i><b>HTML after calling <i>addEnclosingTag( "B", "id=theBold", <b>true</b> )</i></b></i>:
     * &lt;B id=theBold&gt;&lt;SPAN id=span1&gt;Hello!&lt;/SPAN&gt;&lt;/B&gt;
     *      
     * </P>
     *
     * @param strTag The new enclosing tag name
     * @param strAttrs The new enclosing tag's attribute string(s).
     * @param fClose <b>true</b> to also add the closing tag for this tag
     *
     */
    protected final void addEnclosingTag( String strTag, String strAttrs, boolean fClose ){}

    /**
     * Adds an inner tag inside the current tag, but encloses its contents.   This tag will appear between
     * the contents and any existing inner tags.
     * 
     * <P>
     * Example:<br>
     * 
     * <i><b>Original HTML:</b></i> &lt;SPAN id=span1&gt;Hello!&lt;/SPAN&gt;<br><br>
     * <i><b>HTML after calling <i>addInnerTag( "B", "id=theBold", <b>true</b> )</i></b></i>:
     * &lt;SPAN id=span1&gt;&lt;B id=theBold&gt;Hello!&lt;/B&gt;&lt;/SPAN&gt;
     * </P>     
     *
     * @param strTag The new inner tag name
     * @param strAttrs The new inner tag's attribute string(s).
     * @param fClose <b>true</b> to also add the closing tag for this tag
     *
     */
    protected final void addInnerTag( String strTag, String strAttrs, boolean fClose ){}

    /**
     * Adds an attribute inside the current tag's less-than (<) and greater-than (>)
     * characters. 
     *
     * @param strAttr The name of the attribute to add
     * @param strValue The String value of the new attribute        
     */
    protected final void addAttrString( String strAttr, String strValue ){}

    /**
     * Adds an attribute inside the current tag's STYLE attribute.   These styles will only be emitted
     * if <a href="#getSupportsCSS">supportsCSS</a> is <b>true</b>
     *
     * @param strAttr The name of the style to add
     * @param strValue The String value of the new style
     * @see #getSupportsCSS
     * @see #setSupportsCSS
     */
    protected final void addStyleString( String strStyle, String strValue ){}
}