//STRIPPEDSOURCE//
package com.ms.wfc.io;

/**
 * This interface represents a stream that allows reading and writing of data
 * in the form of bytes.
 *
 * @security(checkClassLinking=on)
 */
public interface IByteStream
{
    /**
     * Reads a specified number of bytes from this stream into an array starting at a
     * specified position. 
     * Returns the actual number of bytes read. If the read operation is 
     * successful, the current position of the stream is advanced by the number
     * of bytes read.
     *
     * @param buffer The byte array to read data into.
     * @param index Starting index in the buffer.
     * @param count The number of bytes to read.
     * @return The total number of bytes read into the buffer. This may be less
     * than the number of bytes requested if that many bytes aren't currently
     * available, or zero if the end of the stream is reached.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @see #write
     * @see #canRead
     */
    public int read(byte[] buffer, int index, int count);

    /**
     * Writes a specified number of bytes into this stream from an array starting at a
     * specified position. 
     * If the write operation is successful, the current position of the stream
     * is advanced by <code>count</code>.
     *
     * @param buffer The byte array to write data from.
     * @param index Starting index in the buffer.
     * @param count The number of bytes to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #read
     * @see #canWrite
     */
    public void write(byte[] buffer, int index, int count);

    /**
     * Retrieves the length in bytes of this stream. 
     *
     * @return The length of the stream.
     * @exception IOException If an I/O error occurs or if the length of the
     * stream cannot be determined.
     * @see #setLength
     * @see #canSeek
     */
    public long getLength();

    /**
     * Sets the length of this stream to the specified value.  If the given value
     * is less than the current length of the stream, the stream is truncated.
     * If the given value is larger than the current length of the stream, the
     * stream is expanded. If the stream is expanded, the contents of the
     * stream between the old and the new length are undefined.
     *
     * @param value The new length of the stream.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support resizing.
     * @see #getLength
     * @see #canSeek
     */
    public void setLength(long value);

    /**
     * Retrieves the current position of this stream. 
     *
     * @return The current position of the stream.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support positioning.
     * @see #setPosition
     * @see #canSeek
     */
    public long getPosition();

    /**
     * Sets the current position of this stream to the specified value.  While some
     * streams may support positioning beyond the length of the stream, others
     * will throw an exception in this case.
     *
     * @param value The new position of the stream.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support positioning.
     * @see #getPosition
     * @see #canSeek
     */
    public void setPosition(long value);

    /**
     * Determines whether this stream supports reading.  If a
     * stream does not support reading, calls to the read() method will always
     * raise exceptions.
     *
     * @return true if the stream supports reading, false otherwise.
     * @see #read
     */
    public boolean canRead();

    /**
     * Determines whether this stream supports writing.  If a
     * stream does not support writing, calls to the write() method will always
     * raise exceptions.
     *
     * @return true if the stream supports writing, false otherwise.
     * @see #write
     */
    public boolean canWrite();

    /**
     * Clears all buffers for this stream and causes any buffered data to be written to the
     * underlying device. If a
     * stream does not supports seeking, calls to the getLength(), setLength(),
     * getPosition(), and setPosition() methods will always raise exceptions.
     *
     * @return true if the stream supports seeking, false otherwise.
     * @see #getLength
     * @see #setLength
     * @see #getPosition
     * @see #setPosition
     */
    public boolean canSeek();

    /**
     * Clears all buffers for this stream and causes any buffered data to be written to the
     * underlying device. 
     *
     * @exception IOException If an I/O error occurs.
     */
    public void flush();

    /**
     * Closes this stream and releases any system resources associated with the stream.  Following a call to close(), any operations on the stream may
     * raise exceptions.
     */
    public void close();
}