//STRIPPEDSOURCE//
package com.ms.wfc.io;

import com.ms.com.IStream;

/**
 * This interface represents a stream that allows reading and writing of data
 * in a variety of formats. The interface is an extension of the IByteStream
 * interface.
 *
 * @see com.ms.wfc.io.BufferedStream
 * @see com.ms.wfc.io.ComStreamFromDataStream
 * @see com.ms.wfc.io.DataStream
 * @see com.ms.wfc.io.DataStreamFromComStream
 * @see com.ms.wfc.io.File
 * @see com.ms.wfc.io.IByteStream
 * @see com.ms.wfc.io.IDataStreamProvider
 * @see com.ms.wfc.io.MemoryStream
 *
 * @security(checkClassLinking=on)
 */
public interface IDataStream extends IByteStream
{
    /**
     * Retrieves a COM-compatible IStream object for this stream.  The returned IStream
     * has the same current position and length as this stream, and the streams
     * remain synchronized during subsequent operations on either stream.
     *
     * @return An IStream representation of this stream.
     * @see com.ms.com.IStream
     */
    public IStream getComStream();

    /**
     * Reads a single byte from this stream.  The current
     * position of the stream is advanced by one byte.
     *
     * @return The <code>boolean</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the stream is at the end.
     * @see #writeBoolean
     */
    public boolean readBoolean();

    /**
     * Reads a byte from this stream. 
     *
     * @return The <code>byte</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the stream is at the end.
     * @see #writeByte
     */
    public byte readByte();

    /**
     * Reads a short value from this stream. 
     *
     * @return The <code>short</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before two
     * bytes could be read.
     * @see #writeShort
     */
    public short readShort();

    /**
     * Reads a character from this stream. 
     *
     * @return The <code>char</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before two
     * bytes could be read.
     * @see #writeChar
     */
    public char readChar();

    /**
     * Reads an integer from this stream. 
     *
     * @return The <code>int</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before four
     * bytes could be read.
     * @see #writeInt
     */
    public int readInt();

    /**
     * Reads a long value from this stream. 
     *
     * @return The <code>long</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before eight
     * bytes could be read.
     * @see #writeLong
     */
    public long readLong();

    /**
     * Reads a float value from this stream.
     *
     * @return The <code>float</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before four
     * bytes could be read.
     * @see #writeFloat
     */
    public float readFloat();

    /**
     * Reads a double value from this stream.  
     *
     * @return The <code>double</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached before eight
     * bytes could be read.
     * @see #writeDouble
     */
    public double readDouble();

    /**
     * Reads a length-prefixed Unicode string from this stream.  This method
     * first reads the length of the string as a 32-bit unsigned integer, and
     * then reads that many 16-bit Unicode characters into the string.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeString
     */
    public String readString();

    /**
     * Reads a Unicode string from this stream.  This method reads
     * <code>length</code> 16-bit Unicode characters from the stream and
     * returns a string containing those characters.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeStringChars
     */
    public String readStringChars(int length);

    /**
     * Reads an American National Standards Institute (ANSI) string from this stream.  This method reads
     * <code>length</code> bytes from the stream and converts those bytes to a
     * Unicode string using the system's default code page. The length of the
     * resulting string is a function of the code page and may be less than the
     * number of bytes read from the stream.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeStringCharsAnsi
     */
    public String readStringCharsAnsi(int length);

    /**
     * Reads a null-terminated Unicode string from this stream.  This method
     * keeps reading 16-bit Unicode characters from the stream until a null
     * character is read. The characters (excluding the terminating null
     * character) are then converted to a string.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeStringNull
     */
    public String readStringNull();

    /**
     * Reads a null-terminated ANSI string from this stream.  This method keeps
     * reading bytes from the stream until a null byte is read. The bytes
     * (excluding the terminating null byte) are then converted to a string
     * using the system's default code page.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeStringNullAnsi
     */
    public String readStringNullAnsi();

    /**
     * Reads a string from this stream using a length-prefixed UTF-8 format. 
     * UTF encoded strings are generally more space efficient than Unicode
     * strings, but also take more time to process.
     *
     * @return The <code>String</code> value read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the string.
     * @see #writeUTF
     */
    public String readUTF();

    /**
     * Reads all data from the current position to the end of this stream into a byte array.  If the current position is at the beginning of the stream,
     * this is equivalent to reading the entire stream into a byte array. If
     * the read operation is successful, the current position will be at the
     * end of the stream.
     *
     * @return Byte array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @see com.ms.wfc.io.IByteStream#read
     * @see #readBytes
     */
    public byte[] readToEnd();

    /**
     * Reads the specified number of bytes from this stream into a byte array.  The
     * current position is advanced by <code>count</code> bytes.
     *
     * @param count The number of bytes to read.
     * @return Byte array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see com.ms.wfc.io.IByteStream#read
     * @see #readToEnd
     * @see #writeBytes
     */
    public byte[] readBytes(int count);

    /**
     * Reads the specified number of short integers from this stream into a short integer
     * array.  The current position is advanced by <code>count * 2</code>
     * bytes.
     *
     * @param count The number of short integers to read.
     * @return Short integer array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeShorts
     */
    public short[] readShorts(int count);

    /**
     * Reads the specified number of characters from this stream into a character array.  The current position is advanced by <code>count * 2</code> bytes.
     *
     * @param count The number of characters to read.
     * @return Character array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeChars
     */
    public char[] readChars(int count);

    /**
     * Reads the specified number of integers from this stream into an integer array.  The current position is advanced by <code>count * 4</code> bytes.
     *
     * @param count The number of integers to read.
     * @return Integer array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeInts
     */
    public int[] readInts(int count);

    /**
     * Reads a specified number of long integers from this stream into a long integer array.  The current position is advanced by <code>count * 8</code>
     * bytes.
     *
     * @param count The number of long integers to read.
     * @return Long integer array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeLongs
     */
    public long[] readLongs(int count);

    /**
     * Reads a specified number of float values from this stream into a float
     * array.  The
     * current position is advanced by <code>count * 4</code> bytes.
     *
     * @param count The number of floats to read.
     * @return Float array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeFloats
     */
    public float[] readFloats(int count);

    /**
     * Reads the specified number of double values from this stream into a double
     * array.  The
     * current position is advanced by <code>count * 8</code> bytes.
     *
     * @param count The number of doubles to read.
     * @return Double array containing the data read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeDoubles
     */
    public double[] readDoubles(int count);

    /**
     * Reads the specified number of bytes from this stream into a byte array. 
     * The current position is advanced by <code>count</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of bytes to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeBytes
     */
    public void readBytes(byte[] buffer, int index, int count);

    /**
     * Reads the specified number of short integers from this stream into a short integer
     * array. 
     * The current position is advanced by <code>count * 2</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of short integers to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeShorts
     */
    public void readShorts(short[] buffer, int index, int count);

    /**
     * Reads the specified number of characters from this stream into a character array. 
     * The current position is advanced by <code>count * 2</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of characters to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeChars
     */
    public void readChars(char[] buffer, int index, int count);

    /**
     * Reads the specified number of integers from this stream into an integer array. 
     * The current position is advanced by <code>count * 4</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of integers to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeInts
     */
    public void readInts(int[] buffer, int index, int count);

    /**
     * Reads a specified number of long integers from this stream into a long integer array. 
     * The current position is advanced by <code>count * 8</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of long integers to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeLongs
     */
    public void readLongs(long[] buffer, int index, int count);

    /**
     * Reads a specified number of float values from this stream into a float
     * array. 
     * The current position is advanced by <code>count * 4</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of floats to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeFloats
     */
    public void readFloats(float[] buffer, int index, int count);

    /**
     * Reads the specified number of double values from this stream into a double
     * array. 
     * The current position is advanced by <code>count * 8</code> bytes.
     *
     * @param buffer The array to read data into.
     * @param index Starting index in the array.
     * @param count The number of doubles to read.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support reading.
     * @exception EOFException If the end of the stream is reached while
     * reading the data.
     * @see #writeDoubles
     */
    public void readDoubles(double[] buffer, int index, int count);

    /**
     * Writes a boolean value to this stream.  A single byte is written
     * to the stream with the value 0 representing false or the value 1
     * representing true.
     *
     * @param value The <code>boolean</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readBoolean
     */
    public void writeBoolean(boolean value);

    /**
     * Writes a byte to this stream. 
     *
     * @param value The <code>byte</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readByte
     */
    public void writeByte(byte value);

    /**
     * Writes a short integer to this stream. 
     *
     * @param value The <code>short</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readShort
     */
    public void writeShort(short value);

    /**
     * Writes a character to this stream. 
     *
     * @param value The <code>char</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readChar
     */
    public void writeChar(char value);

    /**
     * Writes an integer value to this stream. 
     *
     * @param value The <code>int</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readInt
     */
    public void writeInt(int value);

    /**
     * Writes a long integer to this stream. 
     *
     * @param value The <code>long</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readLong
     */
    public void writeLong(long value);

    /**
     * Writes a float value to this stream. 
     *
     * @param value The <code>float</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readFloat
     */
    public void writeFloat(float value);

    /**
     * Writes a double value to this stream. 
     *
     * @param value The <code>double</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readDouble
     */
    public void writeDouble(double value);

    /**
     * Writes a length-prefixed Unicode string to this stream.  This method
     * first writes the length of the string as a 32-bit unsigned integer, and
     * then writes that many 16-bit Unicode characters to the stream.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readString
     */
    public void writeString(String value);

    /**
     * Writes a Unicode string to this stream.  The number of Unicode characters
     * written to the stream corresponds to the length of the string.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readStringChars
     */
    public void writeStringChars(String value);

    /**
     * Writes an ANSI string to this stream.  The given string is converted to
     * a sequence of bytes using the system's default code page, and those
     * bytes are then written to the stream. The number of bytes written to the
     * stream is a function of the code page and may be larger than the length
     * of the string.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readStringCharsAnsi
     */
    public void writeStringCharsAnsi(String value);

    /**
     * Writes a null-terminated Unicode string to this stream.  The characters
     * of the string are written to the stream as a sequence of 16-bit Unicode
     * characters followed by a 16-bit null character.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readStringNull
     */
    public void writeStringNull(String value);

    /**
     * Writes a null-terminated ANSI string to this stream.  The given string
     * is converted to a sequence of bytes using the system's default code
     * page, and those bytes are then written to the stream followed by a null
     * byte.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readStringNullAnsi
     */
    public void writeStringNullAnsi(String value);

    /**
     * Writes a string to this stream using a length-prefixed UTF-8 format. 
     * UTF encoded strings are generally more space efficient than Unicode
     * strings, but also take more time to process.
     *
     * @param value The <code>String</code> value to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readUTF
     */
    public void writeUTF(String value);

    /**
     * Writes the contents of the specified byte array to this stream.  The number
     * of bytes written to the stream is <code>buffer.length</code>.
     *
     * @param buffer Byte array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see com.ms.wfc.io.IByteStream#write
     * @see #readBytes
     */
    public void writeBytes(byte[] buffer);

    /**
     * Writes the contents of the specified short integer array to this stream.  The
     * number of bytes written to the stream is <code>buffer.length * 2</code>.
     *
     * @param buffer Short integer array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readShorts
     */
    public void writeShorts(short[] buffer);

    /**
     * Writes the contents of the specified character array to this stream.  The
     * number of bytes written to the stream is <code>buffer.length * 2</code>.
     *
     * @param buffer Character array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readChars
     */
    public void writeChars(char[] buffer);

    /**
     * Writes the contents of the specified integer array to this stream.  The
     * number of bytes written to the stream is <code>buffer.length * 4</code>.
     *
     * @param buffer Integer array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readInts
     */
    public void writeInts(int[] buffer);

    /**
     * Writes the contents of the specified long integer array to this stream.  The
     * number of bytes written to the stream is <code>buffer.length * 8</code>.
     *
     * @param buffer Long integer array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readLongs
     */
    public void writeLongs(long[] buffer);

    /**
     * Writes the contents of the specified float array to this stream.  The number
     * of bytes written to the stream is <code>buffer.length * 4</code>.
     *
     * @param buffer Float array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readFloats
     */
    public void writeFloats(float[] buffer);

    /**
     * Writes the contents of the specified double array to this stream.  The number
     * of bytes written to the stream is <code>buffer.length * 8</code>.
     *
     * @param buffer Double array containing the data to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readDoubles
     */
    public void writeDoubles(double[] buffer);

    /**
     * Writes the contents of the specified short integer array to this stream. 
     * The current position is advanced by <code>count * 2</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of short integers to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readShorts
     */
    public void writeShorts(short[] buffer, int index, int count);

    /**
     * Writes the contents of the specified character array to this stream. 
     * The current position is advanced by <code>count * 2</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of characters to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readChars
     */
    public void writeChars(char[] buffer, int index, int count);

    /**
     * Writes the contents of the specified integer array to this stream. 
     * The current position is advanced by <code>count * 4</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of integers to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readInts
     */
    public void writeInts(int[] buffer, int index, int count);

    /**
     * Writes the contents of the specified long integer array to this stream. 
     * The current position is advanced by <code>count * 8</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of long integers to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readLongs
     */
    public void writeLongs(long[] buffer, int index, int count);

    /**
     * Writes the contents of the specified float array to this stream. 
     * The current position is advanced by <code>count * 4</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of floats to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readFloats
     */
    public void writeFloats(float[] buffer, int index, int count);

    /**
     * Writes the contents of the specified double array to this stream. 
     * The current position is advanced by <code>count * 8</code> bytes.
     *
     * @param buffer The array to write data from.
     * @param index Starting index in the array.
     * @param count The number of doubles to write.
     * @exception IOException If an I/O error occurs or if the stream does not
     * support writing.
     * @see #readDoubles
     */
    public void writeDoubles(double[] buffer, int index, int count);
}