//STRIPPEDSOURCE//
package com.ms.wfc.io;

import com.ms.wfc.core.*;

/**
 * Represents a writer that can write a sequential stream of characters.  A subclass must minimally implement the
 * <code>write(char)</code> method.
 *
 * @see com.ms.wfc.io.Reader
 * @see com.ms.wfc.io.StringWriter
 * @see com.ms.wfc.io.Text
 * @see com.ms.wfc.io.TextWriter
 *
 * @security(checkClassLinking=on)
 */
public abstract class Writer implements IWriter
{
    private String newLine = "\r\n";

    /**
     * Closes this Writer object and releases any system resources associated
     * with the writer.  Following a call to close(), any operations on the Writer may
     * raise exceptions. This default method is empty, but descendant classes
     * can override the method to provide the appropriate functionality.
     */
    public void close() {}

    /**
     * Clears all buffers for this Writer object and causes any buffered data
     * to be written to the underlying device.  This default method is empty, but
     * descendant classes can override the method to provide the appropriate
     * functionality.
     */
    public void flush() {}

    /**
     * Retrieves the line terminator string used by this Writer object. The default line
     * terminator string is a carriage return followed by a line feed ("\r\n").
     *
     * @return The line terminator string.
     * @see #setNewLine
     */
    public String getNewLine() {}

    /**
     * Sets the line terminator string for this Writer object. The line terminator
     * string is written to the text stream whenever one of the
     * <code>writeLine</code> methods are called. In order for text written by
     * the Writer to be readable by a Reader, only one of the following line
     * terminator strings should be used: "\r", "\n", or "\r\n".
     *
     * @param value The line terminator string.
     * @see #getNewLine
     */
    public void setNewLine(String value) {}

    /**
     * Writes the specified item to the text stream.  This default method is empty,
     * but descendant classes can override the method to provide the
     * appropriate functionality.
     *
     * @param value The character to write to the text stream.
     */
    public void write(char value) {}

    /**
     * Writes the specified item to the text stream.  This default method calls
     * <code>write(char)</code> for each of the characters in the character
     * array. If the character array is null, nothing is written.
     *
     * @param buffer The character array to write to the text stream.
     */
    public void write(char[] buffer) {}

    /**
     * Writes the specified item to the text stream.  This method
     * will write <code>count</code> characters of data into this Writer from
     * the <code>buffer</code> character array starting at position
     * <code>index</code>.
     *
     * @param buffer The character array to write data from.
     * @param index Starting index in the buffer.
     * @param count The number of characters to write.
     */
    public void write(char[] buffer, int index, int count) {}

    /**
     * Writes the specified item to the text stream.  This
     * method outputs either the string "true" or the string "false".
     *
     * @param value The boolean to write.
     */
    public void write(boolean value) {}

    /**
     * Writes the specified item to the text stream.  The
     * text representation of the given value is produced by calling the
     * <code>Integer.toString(int)</code> method.
     *
     * @param value The integer to write.
     */
    public void write(int value) {}

    /**
     * Writes the specified item to the text stream.  The
     * text representation of the given value is produced by calling the
     * <code>Long.toString(long)</code> method.
     *
     * @param value The long to write.
     */
    public void write(long value) {}

    /**
     * Writes the specified item to the text stream.  The
     * text representation of the given value is produced by calling the
     * <code>Float.toString(float)</code> method.
     *
     * @param value The float to write.
     */
    public void write(float value) {}

    /**
     * Writes the specified item to the text stream.  The
     * text representation of the given value is produced by calling the
     * <code>Double.toString(double)</code> method.
     *
     * @param value The double to write.
     */
    public void write(double value) {}

    /**
     * Writes the specified item to the text stream.  If the given string is null, nothing
     * is written to the text stream.
     *
     * @param value The string to write.
     */
    public void write(String value) {}

    /**
     * Writes the specified item to the text stream.  If the
     * given object is null, the string "null" is written to the text stream.
     * Otherwise, the object's <code>toString()</code> method is called to
     * produce the string representation, and the resulting string is then
     * written to the output stream.
     *
     * @param value The object to write.
     */
    public void write(Object value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream.  The default line terminator
     * is a carriage return followed by a line feed ("\r\n"), but this value
     * can be changed using the <code>setNewLine(String)</code> method.
     *
     * @see #setNewLine
     */
    public void writeLine() {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The character to write to the text stream.
     * @see #write(char)
     */
    public void writeLine(char value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The character array to write to the text stream.
     * @see #write(char[])
     */
    public void writeLine(char[] buffer) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param buffer The character array to write data from.
     * @param index Starting index in the buffer.
     * @param count The number of characters to write.
     * @see #write(char[], int, int)
     */
    public void writeLine(char[] buffer, int index, int count) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The boolean to write.
     * @see #write(boolean)
     */
    public void writeLine(boolean value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The integer to write.
     * @see #write(int)
     */
    public void writeLine(int value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The long to write.
     * @see #write(long)
     */
    public void writeLine(long value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The float to write.
     * @see #write(float)
     */
    public void writeLine(float value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The double to write.
     * @see #write(double)
     */
    public void writeLine(double value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The string to write.
     * @see #write(String)
     */
    public void writeLine(String value) {}

    /**
     * Writes the specified item followed by a line terminator to the text stream. 
     *
     * @param value The object to write.
     * @see #write(Object)
     */
    public void writeLine(Object value) {}
}