//STRIPPEDSOURCE//

package com.ms.wfc.ui;

import com.ms.wfc.core.*;

/**
 * This interface represents a design-time selection service that components
 * can use to manipulate a form designer's selection.  You can get
 * this interface by calling <b>getService</b> on a components ComponentSite
 * 
 * @see com.ms.wfc.core.IComponentSite#getService
 *
 * @security(checkClassLinking=on)
 */
public interface ISelectionService {

    /**
     * Assigns a delegate to the selectionChanged event.   A
     * selection changed event will fire every time the user changes the current 
     * list of selected components in the designer.  You should keep processing 
     * during this event to a minimum, as this can effect the overall performance 
     * of the form designer.
     * 
     * @param handler An EventHandler that will be called whenever the current
     *                selection changes.  The <i>sender</i> parameter of the
     *                event will be a reference to the ISelectionService.
     * 
     * @see com.ms.wfc.core.EventHandler
     * @see #removeOnSelectionChanged
     */
    public void addOnSelectionChanged(EventHandler handler);
    
    /**
     * Retrieves a boolean value indicating whether this component is selected.   This is faster
     * than getting the entire list of selelcted components.
     * 
     * @return <b>true</b> if this component is part of the user's current
     *         selection, or <b>false</b> if not.
     */    
    public boolean getComponentSelected();
    
    /**
     * Retrieves an array of components that are currently part of the 
     * user's selection.
     * 
     * @return an array of IComponents that represents the current set of
     *         components the user has selected.
     */
    public IComponent [] getSelectedComponents();
    
    /**
     * Removes the delegate that is assigned to the selectionChanged event. 
     * A selection changed event will fire every time the user changes the
     * current list of selected components.
     * 
     * @param handler An EventHandler that is to be removed from the selection
     *                changed event.
     */
    public void removeOnSelectionChanged(EventHandler handler);
    
    /**
     * Selects a set of components in the Forms Designer.   If the array is <b>null</b> or doesn't contain any
     * components, this will remove any current selection.  You should use
     * this method with care, as you can change the behavior of the form
     * designer's selection model.  
     */
    public void setSelectedComponents(IComponent [] components);
    
    /**
     * Sets the style of the selection frame for this component.   This is useful if your component supports different
     * modes of operation (such as an in-place editing mode and a static
     * design mode).  Where possible, you should leave the selection style
     * as is and use the design-time hit testing feature of the ICustomizer
     * interface to provide features at design time.  The value of <i>style</i> 
     * must be one of the  SelectionStyle enum values.
     * 
     * @see com.ms.wfc.ui.SelectionStyle
     * @see com.ms.wfc.core.ICustomizer
     */
    public void setSelectionStyle(int style);
}