//STRIPPEDSOURCE//
package com.ms.wfc.ui;

import com.ms.wfc.win32.*;
import com.ms.win32.*;

/**
 * Encapsulates a Win32 raster operation.   Win32 allows you to get
 * insanely complicated with the different operations you can do, and most developers largely
 * ignore anything beyond the basic predefined operations.  This class allows you to 
 * create much more complicated operations easily and intuitively.<p>
 *
 * To create a raster operation, you build it up from simpler operations and boolean logic.
 * There are six primative RasterOps you can build from:<p>
 *
 *  RasterOp        Meaning<p>
 *  BLACK           The resulting pixel(s) will be black<p>
 *  BRUSH           The resulting pixel(s) will be the color of the current brush<p>
 *  PEN             The resulting pixel(s) will be the color of the current pen<p>
 *  SOURCE          The resulting pixel(s) will be the color of the source bitmap image<p>
 *  TARGET          The resulting pixel(s) will be the color of the target bitmap image<p>
 *  WHITE           The resulting pixel(s) will be white<p>
 *
 * A source bitmap image is generally one you specify.  The image passed into 
 * Graphics.drawImage is a source bitmap image.  The target bitmap image is the location
 * where you want the final rendering to go, which is generally the screen but may
 * be an image object.  This means that using a raster operation of TARGET only will
 * result in nothing happening.<p>
 *
 * You can combine one or more simple raster operations using boolean logic to create
 * more complex operations.  For example, to draw a line that is always the inverse
 * of the screen color, you would pass the return value of RasterOp.TARGET.invert() into
 * your drawing operation.  Much more complex effects can be created using boolean
 * AND, OR, and XOR operators.<p>
 *
 * @security(checkClassLinking=on)
 */
public class RasterOp {
    private static final short UNUSED = -1;

    /**
     * Represents a raster operation in which the resulting pixels will be black.
     */
    public static final RasterOp BLACK  = new RasterOp(0x00, 0x00);

    /**
     * Represents a raster operation in which the resulting pixels will be the color of the
     * current brush.
     */
    public static final RasterOp BRUSH  = new RasterOp(UNUSED, 0xF0);

    /**
     * Represents a raster operation in which the resulting pixels will be the color of the
     * current pen.
     */
    public static final RasterOp PEN    = new RasterOp(0x0C, UNUSED);

    /**
     * Represents a raster operation in which the resulting pixels will be the color of the
     * source image pixels.
     */
    public static final RasterOp SOURCE = new RasterOp(UNUSED, 0xCC);

    /**
     * Represents a raster operation in which the resulting pixels will be the color of the
     * target image pixels.
     */
    public static final RasterOp TARGET = new RasterOp(0x0A, 0xAA);

    /**
     * Represents a raster operation in which the resulting pixels will be white.
     */
    public static final RasterOp WHITE  = new RasterOp(0x0F, 0xFF);

    private short ternaryOp;
    private short binaryOp;

    
    
    
    
    
    private static final short ternaryOpCodes[] = {
        0x0042, 0x0289, 0x0C89, 0x00AA, 0x0C88, 0x00A9, 0x0865, 0x02C5, 
        0x0F08, 0x0245, 0x0329, 0x0B2A, 0x0324, 0x0B25, 0x08A5, 0x0001, 
        0x0C85, 0x00A6, 0x0868, 0x02C8, 0x0869, 0x02C9, 0x5CCA, 0x1D54, 
        0x0D59, 0x1CC8, 0x06C5, 0x0768, 0x06CA, 0x0766, 0x01A5, 0x0385, 
        0x0F09, 0x0248, 0x0326, 0x0B24, 0x0D55, 0x1CC5, 0x06C8, 0x1868, 
        0x0369, 0x16CA, 0x0CC9, 0x1D58, 0x0784, 0x060A, 0x064A, 0x0E2A, 
        0x032A, 0x0B28, 0x0688, 0x0008, 0x06C4, 0x1864, 0x01A8, 0x0388, 
        0x078A, 0x0604, 0x0644, 0x0E24, 0x004A, 0x18A4, 0x1B24, 0x00EA, 
        0x0F0A, 0x0249, 0x0D5D, 0x1CC4, 0x0328, 0x0B29, 0x06C6, 0x076A, 
        0x0368, 0x16C5, 0x0789, 0x0605, 0x0CC8, 0x1954, 0x0645, 0x0E25, 
        0x0325, 0x0B26, 0x06C9, 0x0764, 0x08A9, 0x0009, 0x01A9, 0x0389, 
        0x0785, 0x0609, 0x0049, 0x18A9, 0x0649, 0x0E29, 0x1B29, 0x00E9, 
        0x0365, 0x16C6, 0x0786, 0x0608, 0x0788, 0x0606, 0x0046, 0x18A8, 
        0x58A6, 0x0145, 0x01E9, 0x178A, 0x01E8, 0x1785, 0x1E28, 0x0C65, 
        0x0CC5, 0x1D5C, 0x0648, 0x0E28, 0x0646, 0x0E26, 0x1B28, 0x00E6, 
        0x01E5, 0x1786, 0x1E29, 0x0C68, 0x1E24, 0x0C69, 0x0955, 0x03C9, 
        0x03E9, 0x0975, 0x0C49, 0x1E04, 0x0C48, 0x1E05, 0x17A6, 0x01C5, 
        0x00C6, 0x1B08, 0x0E06, 0x0666, 0x0E08, 0x0668, 0x1D7C, 0x0CE5, 
        0x0C45, 0x1E08, 0x17A9, 0x01C4, 0x17AA, 0x01C9, 0x0169, 0x588A, 
        0x1888, 0x0066, 0x0709, 0x07A8, 0x0704, 0x07A6, 0x16E6, 0x0345, 
        0x00C9, 0x1B05, 0x0E09, 0x0669, 0x1885, 0x0065, 0x0706, 0x07A5, 
        0x03A9, 0x0189, 0x0029, 0x0889, 0x0744, 0x06E9, 0x0B06, 0x0229, 
        0x0E05, 0x0665, 0x1974, 0x0CE8, 0x070A, 0x07A9, 0x16E9, 0x0348, 
        0x074A, 0x06E6, 0x0B09, 0x0226, 0x1CE4, 0x0D7D, 0x0269, 0x08C9, 
        0x00CA, 0x1B04, 0x1884, 0x006A, 0x0E04, 0x0664, 0x0708, 0x07AA, 
        0x03A8, 0x0184, 0x0749, 0x06E4, 0x0020, 0x0888, 0x0B08, 0x0224, 
        0x0E0A, 0x066A, 0x0705, 0x07A4, 0x1D78, 0x0CE9, 0x16EA, 0x0349, 
        0x0745, 0x06E8, 0x1CE9, 0x0D75, 0x0B04, 0x0228, 0x0268, 0x08C8, 
        0x03A5, 0x0185, 0x0746, 0x06EA, 0x0748, 0x06E5, 0x1CE8, 0x0D79, 
        0x1D74, 0x5CE6, 0x02E9, 0x0849, 0x02E8, 0x0848, 0x0086, 0x0A08, 
        0x0021, 0x0885, 0x0B05, 0x022A, 0x0B0A, 0x0225, 0x0265, 0x08C5, 
        0x02E5, 0x0845, 0x0089, 0x0A09, 0x008A, 0x0A0A, 0x02A9, 0x0062 
    };

    
    private RasterOp(int binaryOp, int ternaryOp) {}

    /**
     * Performs a binary AND operation that combines this RasterOp
     * object with another.
     *
     * @param op The raster operation to AND with.
     * @return a new RasterOp that is the logical AND of the two extsting RasterOps.
     */
    public final RasterOp andWith(RasterOp op) {}

    /**
     * Retrieves the ternary ROP code for this raster operation.
     * @return the ternary ROP code for this raster operation.
     */
    public int getRop() {}

    /**
     * Retrieves the binary ROP2 code for this raster operation. 
     * @return the binary ROP2 code for this raster operation.
     */
    public int getRop2() {}

    /**
     * Retrieves a RasterOp object that represents an inversion of this one. 
     * 
     * @return a new RasterOp that is the logical inverse of this RasterOp.
     */
    public final RasterOp invert() {}

    /**
     * Performs a binary OR operation that combines this RasterOp
     * object with another. 
     *
     * @param op The raster operation to OR with.
     * @return a new RasterOp that is the logical OR of the two extsting RasterOps.
     */
    public final RasterOp orWith(RasterOp op) {}

    /**
     * Performs a binary XOR operation that combines this RasterOp
     * object with another. 
     *
     * @param op The raster operation to XOR with.
     * @return a new RasterOp that is the logical XOR of the two extsting RasterOps.
     */
    public final RasterOp xorWith(RasterOp op) {}
}