//STRIPPEDSOURCE//

package com.ms.wfc.util;

import com.ms.wfc.app.*;

/**
 * The job of this class is to collect and track handle usage in 
 * WFC.  Ideally, a developer should never have to call dispose() on
 * any WFC object.  The problem in making this happen is in objects that
 * are very small to the VM garbage collector, but take up huge amounts
 * of resources to the system.  A good example of this is a Win32 region
 * handle.  To the VM, a Region object is a small six byte object, so there
 * isn't much need to garbage collect it anytime soon.  To Win32, however,
 * a region handle consumes expensive USER and GDI resources.  Ideally we
 * would like to be able to mark an object as "expensive" so it uses a different
 * GC algorythm.  In absence of that, we use the HandleCollector class, which
 * runs a daemon thread to GC when handle usage goes up.
 * 
 * @internalonly
 * @hidden
 *
 * @security(checkClassLinking=on)
 */
public class HandleCollector {
    
    private static Daemon collectorDaemon;
    private static HandleType handleTypes[];
    private static int        handleTypeCount;
    
    /**
     * Adds the given handle to the handle collector.  This keeps the
     * handle on a "hot list" of objects that may need to be garbage 
     * collected.
     * 
     * @param handle The handle to add.
     * @param type   The type of the handle.  This must be a value returned
     *               from a prior call to HandleCollector.registerType().
     * @return the handle passed in.  This allows for syntatic convienence.
     * @see #registerType
     */
    public static int add(int handle, int type) {}
    
    /**
     * Called at shutdown to check for handles that are currently allocated.
     * Normally, there should be none.  This will print a list of all
     * handle leaks.
     */
    public static synchronized void checkLeaks() {}
    
    /**
     * Retrieves the number of outstanding Win32 handles for the
     * given handle type.
     * 
     * @param type The handle type.  This must have been obtained 
     *             through a prior call to registerType.
     * @return the count of outstanding handles within this type.
     * @see #registerType
     */
    public static int getHandleCount(int type) {}
    
    /**
     * Registers a new type of handle with the handle collector.
     * 
     * @param typeName The name of the handle type, such as "GDI", 
     *                 "Window", etc.  The given type must have unique
     *                 create and destroy functions, so that adds and
     *                 removes can be balanced.
     * @param expense  This is a percentage that guages how "expensive" the given
     *                 handle type is to keep around.  Handle types with a higher
     *                 percentage will be collected more often than handle types
     *                 with a lower expense.  This number should be from 0-100
     * @return an integer that identifies the new type.  The value will
     *         always be greater than zero.
     */
    public static synchronized int registerType(String typeName, int expense) {}
    
    /**
     * Removes the given handle from the handle collector.  Removing a
     * handle removes it from our "hot list" of objects that should be
     * frequently garbage collected.
     * 
     * @param handle The handle to remove.
     * @param type   The type of the handle.  This must be a value returned
     *               from a prior call to HandleCollector.registerType().
     * @return <b>true</b> if the handle was found and removed, or <b>false</b>
     *         if the handle wasn't found.
     * @see #registerType
     */
    public static boolean remove(int handle, int type) {}

    
    private static class Daemon {
        private Thread thread;
        
        /**
         * Creates a new Daemon thread monitor.
         */
        public Daemon() {}

        /**
         * Forces a GC if needed.
         */
        public void collect() {}
        
        /**
         * Disposes this daemon object by destroying the thread.
         */
        public synchronized void dispose() {}

        
        private void threadProc() {}
    }
    
    
    private static class HandleType {
        public final String name;
        
        private int threshHold;
        private int handleCount;
        private final int deltaPercent;
        private HandleEntry buckets[];
        
        private static final int BUCKETS = 10;
        
        /**
         * Creates a new handle type.
         * 
         * @param name The name of the handle type.
         * @param expense A percentage guaging how expensive the given
         *                handle type is to keep around.
         */
        public HandleType(String name, int expense) {}

        /**
         * Adds a handle to this handle type for monitoring.
         * 
         * @param handle The handle to add.
         */
        public synchronized void add(int handle) {}
        
        /**
         * Checks and reports leaks for handle monitoring.
         */
        public synchronized void checkLeaks() {}

        /**
         * Called after the collector has finished it's work.  Here,
         * we look at the number of objects currently outstanding
         * and establish a new cleanup threshhold.
         */
        public synchronized void collected() {}
        
        
        private int computeHash(int handle) {}

        /**
         * Retrieves the outstanding handle count for this 
         * handle type.
         * 
         * @return the handle count.
         */
        public synchronized int getHandleCount() {}
        
        /**
         * Determines if this handle type needs a GC pass.
         * 
         * @return <b>true</b> if the handle type needs a GC pass, or
         *         <b>false</b> if not.
         */
        public synchronized boolean needCollection() {}
        
        /**
         * Removes the given handle from our monitor list.
         * 
         * @param handle The handle to remove.
         * @return <b>true</b> if the handle was found and removed, or
         *         <b>false</b> if the handle was not in our list.
         */
        public synchronized boolean remove(int handle) {}
        
        
        private static class HandleEntry {
            public final int handle;
            public HandleEntry next;
            
            #if DEBUG
            public final String callStack;
            #endif 

            /**
             * Creates a new handle entry
             * 
             * @param handle The handle to create the entry for.
             */
            public HandleEntry(HandleEntry next, int handle) {}

            /**
             * Converts this handle to a printable string.  the string consists
             * of the handle value along with the callstack for it's
             * allocation.
             * 
             * @return the string representation of this handle.
             */
            public String toString() {}

            #if DEBUG
            
            private static class StackParser {
                String releventStack;
                int startIndex;
                int endIndex;
                int length;

                /**
                 * Creates a new stackparser with the given callstack
                 */
                public StackParser(String callStack) {}

                
                private static boolean containsString(String string, String token) {}
                
                /**
                 * Discards the next line of the stack trace.
                 */
                public void discardNext() {}

                /**
                 * Discards all lines up to and including the line that contains
                 * <i>discardText</i>.
                 */
                public void discardTo(String discardText) {}
                
                
                private String getLine() {}
                
                /**
                 * Rereives the string of the parsed stack trace
                 */
                public String toString() {}
                
                /**
                 * Truncates the stack trace, saving the given # of lines.
                 */
                public void truncate(int lines) {}
            }
            #endif 
        }
    }
}